
from ..parserApiClient import createRequest, get_help

def mutation_policy_parse(mutation_subparsers):
	mutation_policy_parser = mutation_subparsers.add_parser('policy', 
			help='policy() mutation operation', 
			usage=get_help("mutation_policy"))

	mutation_policy_subparsers = mutation_policy_parser.add_subparsers()

	mutation_policy_internetFirewall_parser = mutation_policy_subparsers.add_parser('internetFirewall', 
			help='internetFirewall() policy operation', 
			usage=get_help("mutation_policy_internetFirewall"))

	mutation_policy_internetFirewall_subparsers = mutation_policy_internetFirewall_parser.add_subparsers()

	mutation_policy_internetFirewall_addRule_parser = mutation_policy_internetFirewall_subparsers.add_parser('addRule', 
			help='addRule() internetFirewall operation', 
			usage=get_help("mutation_policy_internetFirewall_addRule"))

	mutation_policy_internetFirewall_addRule_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_internetFirewall_addRule_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_internetFirewall_addRule_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_internetFirewall_addRule_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_internetFirewall_addRule_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_internetFirewall_addRule_parser.set_defaults(func=createRequest,operation_name='mutation.policy.internetFirewall.addRule')

	mutation_policy_internetFirewall_addSection_parser = mutation_policy_internetFirewall_subparsers.add_parser('addSection', 
			help='addSection() internetFirewall operation', 
			usage=get_help("mutation_policy_internetFirewall_addSection"))

	mutation_policy_internetFirewall_addSection_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_internetFirewall_addSection_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_internetFirewall_addSection_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_internetFirewall_addSection_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_internetFirewall_addSection_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_internetFirewall_addSection_parser.set_defaults(func=createRequest,operation_name='mutation.policy.internetFirewall.addSection')

	mutation_policy_internetFirewall_createPolicyRevision_parser = mutation_policy_internetFirewall_subparsers.add_parser('createPolicyRevision', 
			help='createPolicyRevision() internetFirewall operation', 
			usage=get_help("mutation_policy_internetFirewall_createPolicyRevision"))

	mutation_policy_internetFirewall_createPolicyRevision_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_internetFirewall_createPolicyRevision_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_internetFirewall_createPolicyRevision_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_internetFirewall_createPolicyRevision_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_internetFirewall_createPolicyRevision_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_internetFirewall_createPolicyRevision_parser.set_defaults(func=createRequest,operation_name='mutation.policy.internetFirewall.createPolicyRevision')

	mutation_policy_internetFirewall_discardPolicyRevision_parser = mutation_policy_internetFirewall_subparsers.add_parser('discardPolicyRevision', 
			help='discardPolicyRevision() internetFirewall operation', 
			usage=get_help("mutation_policy_internetFirewall_discardPolicyRevision"))

	mutation_policy_internetFirewall_discardPolicyRevision_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_internetFirewall_discardPolicyRevision_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_internetFirewall_discardPolicyRevision_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_internetFirewall_discardPolicyRevision_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_internetFirewall_discardPolicyRevision_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_internetFirewall_discardPolicyRevision_parser.set_defaults(func=createRequest,operation_name='mutation.policy.internetFirewall.discardPolicyRevision')

	mutation_policy_internetFirewall_moveRule_parser = mutation_policy_internetFirewall_subparsers.add_parser('moveRule', 
			help='moveRule() internetFirewall operation', 
			usage=get_help("mutation_policy_internetFirewall_moveRule"))

	mutation_policy_internetFirewall_moveRule_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_internetFirewall_moveRule_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_internetFirewall_moveRule_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_internetFirewall_moveRule_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_internetFirewall_moveRule_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_internetFirewall_moveRule_parser.set_defaults(func=createRequest,operation_name='mutation.policy.internetFirewall.moveRule')

	mutation_policy_internetFirewall_moveSection_parser = mutation_policy_internetFirewall_subparsers.add_parser('moveSection', 
			help='moveSection() internetFirewall operation', 
			usage=get_help("mutation_policy_internetFirewall_moveSection"))

	mutation_policy_internetFirewall_moveSection_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_internetFirewall_moveSection_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_internetFirewall_moveSection_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_internetFirewall_moveSection_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_internetFirewall_moveSection_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_internetFirewall_moveSection_parser.set_defaults(func=createRequest,operation_name='mutation.policy.internetFirewall.moveSection')

	mutation_policy_internetFirewall_publishPolicyRevision_parser = mutation_policy_internetFirewall_subparsers.add_parser('publishPolicyRevision', 
			help='publishPolicyRevision() internetFirewall operation', 
			usage=get_help("mutation_policy_internetFirewall_publishPolicyRevision"))

	mutation_policy_internetFirewall_publishPolicyRevision_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_internetFirewall_publishPolicyRevision_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_internetFirewall_publishPolicyRevision_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_internetFirewall_publishPolicyRevision_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_internetFirewall_publishPolicyRevision_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_internetFirewall_publishPolicyRevision_parser.set_defaults(func=createRequest,operation_name='mutation.policy.internetFirewall.publishPolicyRevision')

	mutation_policy_internetFirewall_removeRule_parser = mutation_policy_internetFirewall_subparsers.add_parser('removeRule', 
			help='removeRule() internetFirewall operation', 
			usage=get_help("mutation_policy_internetFirewall_removeRule"))

	mutation_policy_internetFirewall_removeRule_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_internetFirewall_removeRule_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_internetFirewall_removeRule_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_internetFirewall_removeRule_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_internetFirewall_removeRule_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_internetFirewall_removeRule_parser.set_defaults(func=createRequest,operation_name='mutation.policy.internetFirewall.removeRule')

	mutation_policy_internetFirewall_removeSection_parser = mutation_policy_internetFirewall_subparsers.add_parser('removeSection', 
			help='removeSection() internetFirewall operation', 
			usage=get_help("mutation_policy_internetFirewall_removeSection"))

	mutation_policy_internetFirewall_removeSection_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_internetFirewall_removeSection_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_internetFirewall_removeSection_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_internetFirewall_removeSection_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_internetFirewall_removeSection_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_internetFirewall_removeSection_parser.set_defaults(func=createRequest,operation_name='mutation.policy.internetFirewall.removeSection')

	mutation_policy_internetFirewall_updatePolicy_parser = mutation_policy_internetFirewall_subparsers.add_parser('updatePolicy', 
			help='updatePolicy() internetFirewall operation', 
			usage=get_help("mutation_policy_internetFirewall_updatePolicy"))

	mutation_policy_internetFirewall_updatePolicy_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_internetFirewall_updatePolicy_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_internetFirewall_updatePolicy_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_internetFirewall_updatePolicy_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_internetFirewall_updatePolicy_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_internetFirewall_updatePolicy_parser.set_defaults(func=createRequest,operation_name='mutation.policy.internetFirewall.updatePolicy')

	mutation_policy_internetFirewall_updateRule_parser = mutation_policy_internetFirewall_subparsers.add_parser('updateRule', 
			help='updateRule() internetFirewall operation', 
			usage=get_help("mutation_policy_internetFirewall_updateRule"))

	mutation_policy_internetFirewall_updateRule_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_internetFirewall_updateRule_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_internetFirewall_updateRule_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_internetFirewall_updateRule_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_internetFirewall_updateRule_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_internetFirewall_updateRule_parser.set_defaults(func=createRequest,operation_name='mutation.policy.internetFirewall.updateRule')

	mutation_policy_internetFirewall_updateSection_parser = mutation_policy_internetFirewall_subparsers.add_parser('updateSection', 
			help='updateSection() internetFirewall operation', 
			usage=get_help("mutation_policy_internetFirewall_updateSection"))

	mutation_policy_internetFirewall_updateSection_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_internetFirewall_updateSection_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_internetFirewall_updateSection_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_internetFirewall_updateSection_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_internetFirewall_updateSection_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_internetFirewall_updateSection_parser.set_defaults(func=createRequest,operation_name='mutation.policy.internetFirewall.updateSection')

	mutation_policy_wanFirewall_parser = mutation_policy_subparsers.add_parser('wanFirewall', 
			help='wanFirewall() policy operation', 
			usage=get_help("mutation_policy_wanFirewall"))

	mutation_policy_wanFirewall_subparsers = mutation_policy_wanFirewall_parser.add_subparsers()

	mutation_policy_wanFirewall_addRule_parser = mutation_policy_wanFirewall_subparsers.add_parser('addRule', 
			help='addRule() wanFirewall operation', 
			usage=get_help("mutation_policy_wanFirewall_addRule"))

	mutation_policy_wanFirewall_addRule_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_wanFirewall_addRule_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_wanFirewall_addRule_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_wanFirewall_addRule_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_wanFirewall_addRule_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_wanFirewall_addRule_parser.set_defaults(func=createRequest,operation_name='mutation.policy.wanFirewall.addRule')

	mutation_policy_wanFirewall_addSection_parser = mutation_policy_wanFirewall_subparsers.add_parser('addSection', 
			help='addSection() wanFirewall operation', 
			usage=get_help("mutation_policy_wanFirewall_addSection"))

	mutation_policy_wanFirewall_addSection_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_wanFirewall_addSection_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_wanFirewall_addSection_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_wanFirewall_addSection_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_wanFirewall_addSection_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_wanFirewall_addSection_parser.set_defaults(func=createRequest,operation_name='mutation.policy.wanFirewall.addSection')

	mutation_policy_wanFirewall_createPolicyRevision_parser = mutation_policy_wanFirewall_subparsers.add_parser('createPolicyRevision', 
			help='createPolicyRevision() wanFirewall operation', 
			usage=get_help("mutation_policy_wanFirewall_createPolicyRevision"))

	mutation_policy_wanFirewall_createPolicyRevision_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_wanFirewall_createPolicyRevision_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_wanFirewall_createPolicyRevision_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_wanFirewall_createPolicyRevision_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_wanFirewall_createPolicyRevision_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_wanFirewall_createPolicyRevision_parser.set_defaults(func=createRequest,operation_name='mutation.policy.wanFirewall.createPolicyRevision')

	mutation_policy_wanFirewall_discardPolicyRevision_parser = mutation_policy_wanFirewall_subparsers.add_parser('discardPolicyRevision', 
			help='discardPolicyRevision() wanFirewall operation', 
			usage=get_help("mutation_policy_wanFirewall_discardPolicyRevision"))

	mutation_policy_wanFirewall_discardPolicyRevision_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_wanFirewall_discardPolicyRevision_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_wanFirewall_discardPolicyRevision_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_wanFirewall_discardPolicyRevision_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_wanFirewall_discardPolicyRevision_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_wanFirewall_discardPolicyRevision_parser.set_defaults(func=createRequest,operation_name='mutation.policy.wanFirewall.discardPolicyRevision')

	mutation_policy_wanFirewall_moveRule_parser = mutation_policy_wanFirewall_subparsers.add_parser('moveRule', 
			help='moveRule() wanFirewall operation', 
			usage=get_help("mutation_policy_wanFirewall_moveRule"))

	mutation_policy_wanFirewall_moveRule_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_wanFirewall_moveRule_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_wanFirewall_moveRule_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_wanFirewall_moveRule_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_wanFirewall_moveRule_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_wanFirewall_moveRule_parser.set_defaults(func=createRequest,operation_name='mutation.policy.wanFirewall.moveRule')

	mutation_policy_wanFirewall_moveSection_parser = mutation_policy_wanFirewall_subparsers.add_parser('moveSection', 
			help='moveSection() wanFirewall operation', 
			usage=get_help("mutation_policy_wanFirewall_moveSection"))

	mutation_policy_wanFirewall_moveSection_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_wanFirewall_moveSection_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_wanFirewall_moveSection_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_wanFirewall_moveSection_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_wanFirewall_moveSection_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_wanFirewall_moveSection_parser.set_defaults(func=createRequest,operation_name='mutation.policy.wanFirewall.moveSection')

	mutation_policy_wanFirewall_publishPolicyRevision_parser = mutation_policy_wanFirewall_subparsers.add_parser('publishPolicyRevision', 
			help='publishPolicyRevision() wanFirewall operation', 
			usage=get_help("mutation_policy_wanFirewall_publishPolicyRevision"))

	mutation_policy_wanFirewall_publishPolicyRevision_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_wanFirewall_publishPolicyRevision_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_wanFirewall_publishPolicyRevision_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_wanFirewall_publishPolicyRevision_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_wanFirewall_publishPolicyRevision_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_wanFirewall_publishPolicyRevision_parser.set_defaults(func=createRequest,operation_name='mutation.policy.wanFirewall.publishPolicyRevision')

	mutation_policy_wanFirewall_removeRule_parser = mutation_policy_wanFirewall_subparsers.add_parser('removeRule', 
			help='removeRule() wanFirewall operation', 
			usage=get_help("mutation_policy_wanFirewall_removeRule"))

	mutation_policy_wanFirewall_removeRule_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_wanFirewall_removeRule_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_wanFirewall_removeRule_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_wanFirewall_removeRule_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_wanFirewall_removeRule_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_wanFirewall_removeRule_parser.set_defaults(func=createRequest,operation_name='mutation.policy.wanFirewall.removeRule')

	mutation_policy_wanFirewall_removeSection_parser = mutation_policy_wanFirewall_subparsers.add_parser('removeSection', 
			help='removeSection() wanFirewall operation', 
			usage=get_help("mutation_policy_wanFirewall_removeSection"))

	mutation_policy_wanFirewall_removeSection_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_wanFirewall_removeSection_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_wanFirewall_removeSection_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_wanFirewall_removeSection_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_wanFirewall_removeSection_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_wanFirewall_removeSection_parser.set_defaults(func=createRequest,operation_name='mutation.policy.wanFirewall.removeSection')

	mutation_policy_wanFirewall_updatePolicy_parser = mutation_policy_wanFirewall_subparsers.add_parser('updatePolicy', 
			help='updatePolicy() wanFirewall operation', 
			usage=get_help("mutation_policy_wanFirewall_updatePolicy"))

	mutation_policy_wanFirewall_updatePolicy_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_wanFirewall_updatePolicy_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_wanFirewall_updatePolicy_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_wanFirewall_updatePolicy_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_wanFirewall_updatePolicy_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_wanFirewall_updatePolicy_parser.set_defaults(func=createRequest,operation_name='mutation.policy.wanFirewall.updatePolicy')

	mutation_policy_wanFirewall_updateRule_parser = mutation_policy_wanFirewall_subparsers.add_parser('updateRule', 
			help='updateRule() wanFirewall operation', 
			usage=get_help("mutation_policy_wanFirewall_updateRule"))

	mutation_policy_wanFirewall_updateRule_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_wanFirewall_updateRule_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_wanFirewall_updateRule_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_wanFirewall_updateRule_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_wanFirewall_updateRule_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_wanFirewall_updateRule_parser.set_defaults(func=createRequest,operation_name='mutation.policy.wanFirewall.updateRule')

	mutation_policy_wanFirewall_updateSection_parser = mutation_policy_wanFirewall_subparsers.add_parser('updateSection', 
			help='updateSection() wanFirewall operation', 
			usage=get_help("mutation_policy_wanFirewall_updateSection"))

	mutation_policy_wanFirewall_updateSection_parser.add_argument('accountID', help='The Account ID.')
	mutation_policy_wanFirewall_updateSection_parser.add_argument('json', help='Variables in JSON format.')
	mutation_policy_wanFirewall_updateSection_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_policy_wanFirewall_updateSection_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_policy_wanFirewall_updateSection_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_policy_wanFirewall_updateSection_parser.set_defaults(func=createRequest,operation_name='mutation.policy.wanFirewall.updateSection')



#!/usr/bin/python
import catolib
import logging
import json

############ ENV Settings ############
logging.basicConfig(filename="download-schema.log", filemode='w', format='%(name)s - %(levelname)s - %(message)s')
options = catolib.initParser()

def run():
	# ########################## OPTION 1 ##############################
	# ## Uncomment this secion to build cli as one continuous job 
	# ##################################################################
	# query = {
	# 	'query':'query IntrospectionQuery { __schema { description } }',
	# 	'operationName':'IntrospectionQuery'
	# }
	# success,introspection = catolib.send(options.api_key,query)
	# catolib.parseSchema(introspection)
	# catolib.writeCliDriver(catolib.catoApiSchema)
	# catolib.writeOperationParsers(catolib.catoApiSchema)
	# catolib.writeReadmes(catolib.catoApiSchema)
	# ##################################################################

	######################### OPTION 2 ##############################
	## Uncomment to manually run each step manually writing files locally 
	######################### Step 1 ################################ 
	## Run intrpsection to pull down schema and write locally
	################################################################# 
	query = {
		'query':'query IntrospectionQuery { __schema { description } }',
		'operationName':'IntrospectionQuery'
	}
	success,resp = catolib.send(options.api_key,query)
	catolib.writeFile("introspection.json",json.dumps(resp, indent=4, sort_keys=True))
	# ################################################################# 
	
	######################### Step 2 ################################ 
	## Load from introsspeection from local file, and 
	## use catolib to parse the schema, write locally 
	## to catoApiIntrospection.json and catoApiSchema.json
	################################################################# 
	introspection = catolib.loadJSON("introspection.json")
	catolib.parseSchema(introspection)
	catolib.writeFile("catoApiIntrospection.json",json.dumps(catolib.catoApiIntrospection, indent=4, sort_keys=True))
	catolib.writeFile("catoApiSchema.json",json.dumps(catolib.catoApiSchema, indent=4, sort_keys=True))
	#################################################################      

	####################### Step 3 ################################ 
	## Load catoApiSchema from local file, and write cliDriver
	## operation parsers, and readmes locally
	#############################################################
	catolib.catoApiSchema = catolib.loadJSON("catoApiSchema.json")
	catolib.writeCliDriver(catolib.catoApiSchema)
	catolib.writeOperationParsers(catolib.catoApiSchema)
	catolib.writeReadmes(catolib.catoApiSchema)
	###############################################################

if __name__ == '__main__':
	run()



import catocli.parsers.custom.customLib as customLib

def custom_parse(subparsers):
	entityTypes = ["account","admin","allocatedIP","any","availablePooledUsage","availableSiteUsage","dhcpRelayGroup","groupSubscription","host","lanFirewall","localRouting","location","mailingListSubscription","networkInterface","portProtocol","simpleService","site","siteRange","timezone","vpnUser","webhookSubscription"]
	entity_parser = subparsers.add_parser('entity', help='Entity Lookup', usage='catocli entity <operationName> [options]')
	entity_subparsers = entity_parser.add_subparsers(description='valid subcommands', help='additional help')

	for entity in entityTypes:
		item_parser = entity_subparsers.add_parser(entity, help="entityLookup() for type: "+entity, usage='catocli entity '+entity+' <operationName> [options]')
		item_subparsers = item_parser.add_subparsers(description='valid subcommands', help='additional help')

		item_list_parser = item_subparsers.add_parser('list', 
				help='entity '+entity+' list', 
				usage=get_help_custom("entity_"+entity+"_list"))

		item_list_parser.add_argument('accountID', help='The Account ID.')
		item_list_parser.add_argument('-s', help='Search string', default='', nargs='?')
		item_list_parser.add_argument('-f', default="csv", choices=["json","csv"], nargs='?', 
			help='Specify format for output')
		item_list_parser.add_argument('-t', const=True, default=False, nargs='?', 
			help='Print test request preview without sending api call')
		item_list_parser.add_argument('-v', const=True, default=False, nargs='?', 
			help='Verbose output')
		item_list_parser.add_argument('-p', const=True, default=False, nargs='?', 
			help='Pretty print')
		
		item_list_parser.set_defaults(func=customLib.entityTypeList,operation_name=entity)

def get_help_custom(path):
	matchCmd = "catocli "+path.replace("_"," ")
	import os
	pwd = os.path.dirname(__file__)
	# doc = path+"/README.md"
	abs_path = os.path.join(pwd, "README.md")
	new_line = "\nEXAMPLES:\n"
	lines = open(abs_path, "r").readlines()
	for line in lines:
		if f"{matchCmd}" in line:
			clean_line = line.replace("<br /><br />", "").replace("`","")
			new_line += f"{clean_line}\n"
	# matchArg = path.replace("_",".")
	# for line in lines:
	# 	if f"`{matchArg}" in line:
	# 		clean_line = line.replace("<br /><br />", "").replace("`","")
	# 		new_line += f"{clean_line}\n"
	return new_line



# coding: utf-8

from __future__ import absolute_import
import re
import json

# python 2 and python 3 compatibility library
import sys
sys.path.insert(0, 'vendor')
import six
from graphql_client.api_client_types import ApiClient

class CallApi(object):
	def __init__(self, api_client=None):
		if api_client is None:
			api_client = ApiClient()
		self.api_client = api_client

	def call_api(self, body, args, **kwargs):  # noqa: E501
		(data) = self.call_api_with_http_info(body, args, **kwargs)  # noqa: E501
		return data
	
	def call_api_with_http_info(self, body, args, **kwargs):  # noqa: E501
		all_params = ['body', 'sync_type']  # noqa: E501
		all_params.append('async_req')
		if args.get("v")==True:
			all_params.append('_return_http_data_only')
			all_params.append('_preload_content')
			all_params.append('_request_timeout')

		params = locals()
		for key, val in six.iteritems(params['kwargs']):
			if key not in all_params:
				raise TypeError(
					"Got an unexpected keyword argument '%s'"
					" to method create_asset4" % key
				)
			params[key] = val
		del params['kwargs']

		if ('body' not in params or params['body'] is None):
			raise ValueError("Missing the required parameter `body` when calling `create_asset4`")  

		collection_formats = {}

		header_params = {}

		form_params = []
		local_var_files = {}

		body_params = None
		if 'body' in params:
			body_params = params['body']
		header_params['Accept'] = self.api_client.select_header_accept(['application/json'])
		header_params['Content-Type'] = self.api_client.select_header_content_type(['application/json'])
		header_params['x-api-key'] = self.api_client.configuration.api_key['x-api-key']
		header_params['User-Agent'] = "Cato-CLI-v"+self.api_client.configuration.version

		if args.get("v")==True:
			print("Host: ",self.api_client.configuration.host)
			print("Request Headers:",json.dumps(header_params,indent=4,sort_keys=True))
			print("Request Data:",json.dumps(body_params,indent=4,sort_keys=True),"\n\n")
		
		return self.api_client.call_api(
			header_params,
			body=body_params,
			files=local_var_files,
			response_type="NoSchema",  # noqa: E501 
			async_req=params.get('async_req'),
			_return_http_data_only=params.get('_return_http_data_only'),
			_preload_content=params.get('_preload_content', True),
			_request_timeout=params.get('_request_timeout'),
			collection_formats=collection_formats)



# coding: utf-8

from __future__ import absolute_import

import datetime
import json
import mimetypes
from multiprocessing.pool import ThreadPool
import os
import re
import tempfile

from graphql_client.configuration import Configuration
import graphql_client.models
from graphql_client import api_client

# python 2 and python 3 compatibility library
import sys
sys.path.insert(0, 'vendor')
import six

class ApiClient(object):
	PRIMITIVE_TYPES = (float, bool, bytes, six.text_type) + six.integer_types
	NATIVE_TYPES_MAPPING = {
		'int': int,
		'long': int if six.PY3 else long,
		'float': float,
		'str': str,
		'bool': bool,
		'date': datetime.date,
		'datetime': datetime.datetime,
		'object': object,
	}

	def __init__(self, configuration=None, header_name=None, header_value=None,
				 cookie=None):
		if configuration is None:
			configuration = Configuration()
		self.configuration = configuration

		self.pool = ThreadPool()
		self.rest_client = api_client.QUERYClientObject(configuration)
		self.default_headers = {}
		if header_name is not None:
			self.default_headers[header_name] = header_value
		self.cookie = cookie
		self.user_agent = "Cato-CLI-v"+self.configuration.version

	def __del__(self):
		self.pool.close()
		self.pool.join()

	@property
	def user_agent(self):
		"""User agent for this API client"""
		return self.default_headers['User-Agent']

	@user_agent.setter
	def user_agent(self, value):
		self.default_headers['User-Agent'] = value

	def set_default_header(self, header_name, header_value):
		self.default_headers[header_name] = header_value

	def __call_api(
			self, header_params=None, body=None,
			files=None, response_type=None,
			_return_http_data_only=None, collection_formats=None,
			_preload_content=True, _request_timeout=None):

		config = self.configuration

		# header parameters
		header_params = header_params or {}
		header_params.update(self.default_headers)
		if self.cookie:
			header_params['Cookie'] = self.cookie
		if header_params:
			header_params = self.sanitize_for_serialization(header_params)
			header_params = dict(self.parameters_to_tuples(header_params,
														   collection_formats))

		# body
		if body:
			body = self.sanitize_for_serialization(body)

		# request url
		url = self.configuration.host

		# perform request and return response
		response_data = self.request(
			 url, headers=header_params, body=body,
			_preload_content=_preload_content,
			_request_timeout=_request_timeout)

		self.last_response = response_data

		return_data = response_data
		if _preload_content:
			# deserialize response data
			if response_type:
				return_data = self.deserialize(response_data, response_type)
			else:
				return_data = None

		if _return_http_data_only:
			return (return_data)
		else:
			return (return_data, response_data.status,
					response_data.getheaders())

	def sanitize_for_serialization(self, obj):
		"""Builds a JSON POST object.
		If obj is None, return None.
		If obj is str, int, long, float, bool, return directly.
		If obj is datetime.datetime, datetime.date
			convert to string in iso8601 format.
		If obj is list, sanitize each element in the list.
		If obj is dict, return the dict.
		:param obj: The data to serialize.
		:return: The serialized form of data.
		"""
		if obj is None:
			return None
		elif isinstance(obj, self.PRIMITIVE_TYPES):
			return obj
		elif isinstance(obj, list):
			return [self.sanitize_for_serialization(sub_obj)
					for sub_obj in obj]
		elif isinstance(obj, tuple):
			return tuple(self.sanitize_for_serialization(sub_obj)
						 for sub_obj in obj)
		elif isinstance(obj, (datetime.datetime, datetime.date)):
			return obj.isoformat()

		if isinstance(obj, dict):
			obj_dict = obj
		else:
			obj_dict = {obj.attribute_map[attr]: getattr(obj, attr)
						for attr, _ in six.iteritems(obj.swagger_types)
						if getattr(obj, attr) is not None}

		return {key: self.sanitize_for_serialization(val)
				for key, val in six.iteritems(obj_dict)}

	def deserialize(self, response, response_type):
		# handle file downloading
		# save response body into a tmp file and return the instance
		if response_type == "file":
			return self.__deserialize_file(response)

		# fetch data from response object
		try:
			data = json.loads(response.data)
		except ValueError:
			data = response.data

		return self.__deserialize(data, response_type)

	def __deserialize(self, data, klass):
		if data is None:
			return None

		if type(klass) == str:
			if klass.startswith('list['):
				sub_kls = re.match(r'list\[(.*)\]', klass).group(1)
				return [self.__deserialize(sub_data, sub_kls)
						for sub_data in data]

			if klass.startswith('dict('):
				sub_kls = re.match(r'dict\(([^,]*), (.*)\)', klass).group(2)
				return {k: self.__deserialize(v, sub_kls)
						for k, v in six.iteritems(data)}

			# convert str to class
			if klass in self.NATIVE_TYPES_MAPPING:
				klass = self.NATIVE_TYPES_MAPPING[klass]
			else:
				klass = getattr(graphql_client.models, klass)

		if klass in self.PRIMITIVE_TYPES:
			return self.__deserialize_primitive(data, klass)
		elif klass == object:
			return self.__deserialize_object(data)
		elif klass == datetime.date:
			return self.__deserialize_date(data)
		elif klass == datetime.datetime:
			return self.__deserialize_datatime(data)
		else:
			return self.__deserialize_model(data, klass)

	def call_api(self, header_params=None, body=None, files=None,
				 response_type=None, async_req=None,
				 _return_http_data_only=None, collection_formats=None,
				 _preload_content=True, _request_timeout=None):

		if not async_req:
			return self.__call_api( header_params, body, files,
								   response_type, _return_http_data_only, collection_formats,
								   _preload_content, _request_timeout)
		else:
			thread = self.pool.apply_async(self.__call_api, ( header_params, 
										   body, files, response_type, 
										   _return_http_data_only,
										   collection_formats,
										   _preload_content, _request_timeout))
		return thread

	def request(self, url, headers=None, 
			 body=None, _preload_content=True,
			_request_timeout=None):

		return self.rest_client.POST(url, headers=headers,
									 _preload_content=_preload_content,
									 _request_timeout=_request_timeout,
									 body=body)

	def parameters_to_tuples(self, params, collection_formats):
		"""Get parameters as list of tuples, formatting collections.

		:param params: Parameters as dict or list of two-tuples
		:param dict collection_formats: Parameter collection formats
		:return: Parameters as list of tuples, collections formatted
		"""
		new_params = []
		if collection_formats is None:
			collection_formats = {}
		for k, v in six.iteritems(params) if isinstance(params, dict) else params:  # noqa: E501
			if k in collection_formats:
				collection_format = collection_formats[k]
				if collection_format == 'multi':
					new_params.extend((k, value) for value in v)
				else:
					if collection_format == 'ssv':
						delimiter = ' '
					elif collection_format == 'tsv':
						delimiter = '\t'
					elif collection_format == 'pipes':
						delimiter = '|'
					else:  # csv is the default
						delimiter = ','
					new_params.append(
						(k, delimiter.join(str(value) for value in v)))
			else:
				new_params.append((k, v))
		return new_params

	def select_header_accept(self, accepts):
		"""Returns `Accept` based on an array of accepts provided.

		:param accepts: List of headers.
		:return: Accept (e.g. application/json).
		"""
		if not accepts:
			return

		accepts = [x.lower() for x in accepts]

		if 'application/json' in accepts:
			return 'application/json'
		else:
			return ', '.join(accepts)

	def select_header_content_type(self, content_types):
		"""Returns `Content-Type` based on an array of content_types provided.

		:param content_types: List of content-types.
		:return: Content-Type (e.g. application/json).
		"""
		if not content_types:
			return 'application/json'

		content_types = [x.lower() for x in content_types]

		if 'application/json' in content_types or '*/*' in content_types:
			return 'application/json'
		else:
			return content_types[0]

	def __deserialize_file(self, response):
		"""Deserializes body to file

		Saves response body into a file in a temporary folder,
		using the filename from the `Content-Disposition` header if provided.

		:param response:  RESTResponse.
		:return: file path.
		"""
		fd, path = tempfile.mkstemp(dir=self.configuration.temp_folder_path)
		os.close(fd)
		os.remove(path)

		content_disposition = response.getheader("Content-Disposition")
		if content_disposition:
			filename = re.search(r'filename=[\'"]?([^\'"\s]+)[\'"]?',
								 content_disposition).group(1)
			path = os.path.join(os.path.dirname(path), filename)
			response_data = response.data
			with open(path, "wb") as f:
				if isinstance(response_data, str):
					# change str to bytes so we can write it
					response_data = response_data.encode('utf-8')
					f.write(response_data)
				else:
					f.write(response_data)
		return path

	def __deserialize_primitive(self, data, klass):
		"""Deserializes string to primitive type.

		:param data: str.
		:param klass: class literal.

		:return: int, long, float, str, bool.
		"""
		try:
			return klass(data)
		except UnicodeEncodeError:
			return six.text_type(data)
		except TypeError:
			return data

	def __deserialize_object(self, value):
		"""Return a original value.

		:return: object.
		"""
		return value

	def __deserialize_date(self, string):
		"""Deserializes string to date.

		:param string: str.
		:return: date.
		"""
		try:
			from dateutil.parser import parse
			return parse(string).date()
		except ImportError:
			return string
		except ValueError:
			raise apiclient.ApiException(
				status=0,
				reason="Failed to parse `{0}` as date object".format(string)
			)

	def __deserialize_datatime(self, string):
		"""Deserializes string to datetime.

		The string should be in iso8601 datetime format.

		:param string: str.
		:return: datetime.
		"""
		try:
			from dateutil.parser import parse
			return parse(string)
		except ImportError:
			return string
		except ValueError:
			raise apiclient.ApiException(
				status=0,
				reason=(
					"Failed to parse `{0}` as datetime object"
					.format(string)
				)
			)

	def __hasattr(self, object, name):
			return name in object.__class__.__dict__

	def __deserialize_model(self, data, klass):
		"""Deserializes list or dict to model.

		:param data: dict, list.
		:param klass: class literal.
		:return: model object.
		"""

		if not klass.swagger_types and not self.__hasattr(klass, 'get_real_child_model'):
			return data

		kwargs = {}
		if klass.swagger_types is not None:
			for attr, attr_type in six.iteritems(klass.swagger_types):
				if (data is not None and
						klass.attribute_map[attr] in data and
						isinstance(data, (list, dict))):
					value = data[klass.attribute_map[attr]]
					kwargs[attr] = self.__deserialize(value, attr_type)

		instance = klass(**kwargs)

		if (isinstance(instance, dict) and
				klass.swagger_types is not None and
				isinstance(data, dict)):
			for key, value in data.items():
				if key not in klass.swagger_types:
					instance[key] = value
		if self.__hasattr(instance, 'get_real_child_model'):
			klass_name = instance.get_real_child_model(data)
			if klass_name:
				instance = self.__deserialize(data, klass_name)
		return instance




import os
import argparse
import json
import catocli
from graphql_client import Configuration
from graphql_client.api_client import ApiException
from ..parsers.parserApiClient import get_help
import sys
sys.path.insert(0, 'vendor')
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
if "CATO_TOKEN" not in os.environ:
	print("Missing authentication, please set the CATO_TOKEN environment variable with your api key.")
	exit()
CATO_TOKEN = os.getenv("CATO_TOKEN")
CATO_DEBUG = bool(os.getenv("CATO_DEBUG", False))
from ..parsers.raw import raw_parse
from ..parsers.custom import custom_parse
from ..parsers.query_siteLocation import query_siteLocation_parse
from ..parsers.mutation_admin import mutation_admin_parse
from ..parsers.mutation_container import mutation_container_parse
from ..parsers.mutation_policy import mutation_policy_parse
from ..parsers.mutation_site import mutation_site_parse
from ..parsers.mutation_sites import mutation_sites_parse
from ..parsers.query_accountBySubdomain import query_accountBySubdomain_parse
from ..parsers.query_accountMetrics import query_accountMetrics_parse
from ..parsers.query_accountRoles import query_accountRoles_parse
from ..parsers.query_accountSnapshot import query_accountSnapshot_parse
from ..parsers.query_admin import query_admin_parse
from ..parsers.query_admins import query_admins_parse
from ..parsers.query_appStats import query_appStats_parse
from ..parsers.query_appStatsTimeSeries import query_appStatsTimeSeries_parse
from ..parsers.query_auditFeed import query_auditFeed_parse
from ..parsers.query_container import query_container_parse
from ..parsers.query_entityLookup import query_entityLookup_parse
from ..parsers.query_events import query_events_parse
from ..parsers.query_eventsFeed import query_eventsFeed_parse
from ..parsers.query_eventsTimeSeries import query_eventsTimeSeries_parse
from ..parsers.query_hardwareManagement import query_hardwareManagement_parse
from ..parsers.query_licensing import query_licensing_parse
from ..parsers.query_policy import query_policy_parse
from ..parsers.query_subDomains import query_subDomains_parse
from ..parsers.query_xdr import query_xdr_parse

configuration = Configuration()
configuration.verify_ssl = False
configuration.api_key["x-api-key"] = CATO_TOKEN
configuration.host = "{}".format(catocli.__cato_host__)
configuration.debug = CATO_DEBUG
configuration.version = "{}".format(catocli.__version__)

parser = argparse.ArgumentParser(prog='catocli', usage='%(prog)s <operationType> <operationName> [options]', description="CLI for query on CATO via API.")
parser.add_argument('--version', action='version', version=catocli.__version__)
subparsers = parser.add_subparsers()
custom_parsers = custom_parse(subparsers)
raw_parsers = subparsers.add_parser('raw', help='Raw GraphQL', usage=get_help("raw"))
raw_parser = raw_parse(raw_parsers)
query_parser = subparsers.add_parser('query', help='Query', usage='catocli query <operationName> [options]')
query_subparsers = query_parser.add_subparsers(description='valid subcommands', help='additional help')
query_siteLocation_parser = query_siteLocation_parse(query_subparsers)
mutation_parser = subparsers.add_parser('mutation', help='Mutation', usage='catocli mutation <operationName> [options]')
mutation_subparsers = mutation_parser.add_subparsers(description='valid subcommands', help='additional help')

mutation_admin_parser = mutation_admin_parse(mutation_subparsers)
mutation_container_parser = mutation_container_parse(mutation_subparsers)
mutation_policy_parser = mutation_policy_parse(mutation_subparsers)
mutation_site_parser = mutation_site_parse(mutation_subparsers)
mutation_sites_parser = mutation_sites_parse(mutation_subparsers)
query_accountBySubdomain_parser = query_accountBySubdomain_parse(query_subparsers)
query_accountMetrics_parser = query_accountMetrics_parse(query_subparsers)
query_accountRoles_parser = query_accountRoles_parse(query_subparsers)
query_accountSnapshot_parser = query_accountSnapshot_parse(query_subparsers)
query_admin_parser = query_admin_parse(query_subparsers)
query_admins_parser = query_admins_parse(query_subparsers)
query_appStats_parser = query_appStats_parse(query_subparsers)
query_appStatsTimeSeries_parser = query_appStatsTimeSeries_parse(query_subparsers)
query_auditFeed_parser = query_auditFeed_parse(query_subparsers)
query_container_parser = query_container_parse(query_subparsers)
query_entityLookup_parser = query_entityLookup_parse(query_subparsers)
query_events_parser = query_events_parse(query_subparsers)
query_eventsFeed_parser = query_eventsFeed_parse(query_subparsers)
query_eventsTimeSeries_parser = query_eventsTimeSeries_parse(query_subparsers)
query_hardwareManagement_parser = query_hardwareManagement_parse(query_subparsers)
query_licensing_parser = query_licensing_parse(query_subparsers)
query_policy_parser = query_policy_parse(query_subparsers)
query_subDomains_parser = query_subDomains_parse(query_subparsers)
query_xdr_parser = query_xdr_parse(query_subparsers)


def main(args=None):
	args = parser.parse_args(args=args)
	try:
		response = args.func(args, configuration)

		if type(response) == ApiException:
			print("ERROR! Status code: {}".format(response.status))
			print(response)
		else:
			if response!=None:
				print(json.dumps(response[0], sort_keys=True, indent=4))
	except AttributeError:
		print('Missing arguments. Usage: catocli <operation> -h')



from __future__ import annotations

import socket
import typing

from ..exceptions import LocationParseError
from .timeout import _DEFAULT_TIMEOUT, _TYPE_TIMEOUT

_TYPE_SOCKET_OPTIONS = typing.List[typing.Tuple[int, int, typing.Union[int, bytes]]]

if typing.TYPE_CHECKING:
    from .._base_connection import BaseHTTPConnection


def is_connection_dropped(conn: BaseHTTPConnection) -> bool:  # Platform-specific
    """
    Returns True if the connection is dropped and should be closed.
    :param conn: :class:`urllib3.connection.HTTPConnection` object.
    """
    return not conn.is_connected


# This function is copied from socket.py in the Python 2.7 standard
# library test suite. Added to its signature is only `socket_options`.
# One additional modification is that we avoid binding to IPv6 servers
# discovered in DNS if the system doesn't have IPv6 functionality.
def create_connection(
    address: tuple[str, int],
    timeout: _TYPE_TIMEOUT = _DEFAULT_TIMEOUT,
    source_address: tuple[str, int] | None = None,
    socket_options: _TYPE_SOCKET_OPTIONS | None = None,
) -> socket.socket:
    """Connect to *address* and return the socket object.

    Convenience function.  Connect to *address* (a 2-tuple ``(host,
    port)``) and return the socket object.  Passing the optional
    *timeout* parameter will set the timeout on the socket instance
    before attempting to connect.  If no *timeout* is supplied, the
    global default timeout setting returned by :func:`socket.getdefaulttimeout`
    is used.  If *source_address* is set it must be a tuple of (host, port)
    for the socket to bind as a source address before making the connection.
    An host of '' or port 0 tells the OS to use the default.
    """

    host, port = address
    if host.startswith("["):
        host = host.strip("[]")
    err = None

    # Using the value from allowed_gai_family() in the context of getaddrinfo lets
    # us select whether to work with IPv4 DNS records, IPv6 records, or both.
    # The original create_connection function always returns all records.
    family = allowed_gai_family()

    try:
        host.encode("idna")
    except UnicodeError:
        raise LocationParseError(f"'{host}', label empty or too long") from None

    for res in socket.getaddrinfo(host, port, family, socket.SOCK_STREAM):
        af, socktype, proto, canonname, sa = res
        sock = None
        try:
            sock = socket.socket(af, socktype, proto)

            # If provided, set socket level options before connecting.
            _set_socket_options(sock, socket_options)

            if timeout is not _DEFAULT_TIMEOUT:
                sock.settimeout(timeout)
            if source_address:
                sock.bind(source_address)
            sock.connect(sa)
            # Break explicitly a reference cycle
            err = None
            return sock

        except OSError as _:
            err = _
            if sock is not None:
                sock.close()

    if err is not None:
        try:
            raise err
        finally:
            # Break explicitly a reference cycle
            err = None
    else:
        raise OSError("getaddrinfo returns an empty list")


def _set_socket_options(
    sock: socket.socket, options: _TYPE_SOCKET_OPTIONS | None
) -> None:
    if options is None:
        return

    for opt in options:
        sock.setsockopt(*opt)


def allowed_gai_family() -> socket.AddressFamily:
    """This function is designed to work in the context of
    getaddrinfo, where family=socket.AF_UNSPEC is the default and
    will perform a DNS search for both IPv6 and IPv4 records."""

    family = socket.AF_INET
    if HAS_IPV6:
        family = socket.AF_UNSPEC
    return family


def _has_ipv6(host: str) -> bool:
    """Returns True if the system can bind an IPv6 address."""
    sock = None
    has_ipv6 = False

    if socket.has_ipv6:
        # has_ipv6 returns true if cPython was compiled with IPv6 support.
        # It does not tell us if the system has IPv6 support enabled. To
        # determine that we must bind to an IPv6 address.
        # https://github.com/urllib3/urllib3/pull/611
        # https://bugs.python.org/issue658327
        try:
            sock = socket.socket(socket.AF_INET6)
            sock.bind((host, 0))
            has_ipv6 = True
        except Exception:
            pass

    if sock:
        sock.close()
    return has_ipv6


HAS_IPV6 = _has_ipv6("::1")



#!/usr/bin/python
import datetime
import json
import ssl
import sys
import time
import urllib.parse
import urllib.request
import logging
from optparse import OptionParser
import os
import sys
import copy

api_call_count = 0
start = datetime.datetime.now()
catoApiIntrospection = {
	"enums": {},
	"scalars": {},
	"objects": {},
	"input_objects": {},
	"unions": {},
	"interfaces": {},
	"unknowns": {}
}
catoApiSchema = {
	"query": {},
	"mutation": {}
}

def initParser():
	if "CATO_TOKEN" not in os.environ:
		print("Missing authentication, please set the CATO_TOKEN environment variable with your api key.")
		exit()
	
	# Process options
	parser = OptionParser()
	parser.add_option("-I", dest="ID", help="Account ID")
	parser.add_option("-P", dest="prettify", action="store_true", help="Prettify output")
	parser.add_option("-p", dest="print_entities", action="store_true", help="Print entity records")
	parser.add_option("-v", dest="verbose", action="store_true", help="Print debug info")
	(options, args) = parser.parse_args()
	options.api_key = os.getenv("CATO_TOKEN")
	if options.ID is None is None:
		parser.print_help()
		sys.exit()
	if options.verbose:
		logging.getLogger().setLevel(logging.DEBUG)
	else:
		logging.getLogger().setLevel(logging.INFO)
	return options

def loadJSON(file):
	CONFIG = {}
	try:
		with open(file, 'r') as data:
			CONFIG = json.load(data)
			logging.warning("Loaded "+file+" data")
			return CONFIG
	except:
		logging.warning("File \""+file+"\" not found.")
		exit()

def writeFile(fileName, data):
	open(fileName, 'w+').close()
	file=open(fileName,"w+")
	file.write(data)

def openFile(fileName, readMode="rt"):
	try:
		with open(fileName, readMode) as f:
			fileTxt = f.read()
		f.closed
		return fileTxt
	except:
		print('[ERROR] File path "'+fileName+'" in csv not found, or script unable to read.')
		exit()

############ parsing schema ############

def parseSchema(schema):
	mutationOperationsTMP = {}
	queryOperationsTMP = {}
	for i, type in enumerate(schema["data"]["__schema"]["types"]):
		if type["kind"] == "ENUM":
			catoApiIntrospection["enums"][type["name"]] = copy.deepcopy(type)
		elif type["kind"] == "SCALAR":
			catoApiIntrospection["scalars"][type["name"]] = copy.deepcopy(type)
		elif type["kind"] == "INPUT_OBJECT":
			catoApiIntrospection["input_objects"][type["name"]] = copy.deepcopy(type)
		elif type["kind"] == "INTERFACE":
			catoApiIntrospection["interfaces"][type["name"]] = copy.deepcopy(type)
		elif type["kind"] == "UNION":
			catoApiIntrospection["unions"][type["name"]] = copy.deepcopy(type)
		elif type["kind"] == "OBJECT":
			if type["name"] == "Query":
				for field in type["fields"]:
					if field["name"] =="xdr":
						queryOperationsTMP[field["name"]] = copy.deepcopy(field)
					else:
						catoApiSchema["query"]["query."+field["name"]] = copy.deepcopy(field)
						# catoParserMapping["query"][field["name"]]
			elif type["name"] == "Mutation":
				for field in type["fields"]:
					mutationOperationsTMP[field["name"]] = copy.deepcopy(field)
			else:
				catoApiIntrospection["objects"][type["name"]] = copy.deepcopy(type)
	
	for queryType in queryOperationsTMP:
		parentQueryOperationType = copy.deepcopy(queryOperationsTMP[queryType])
		getChildOperations("query", parentQueryOperationType, parentQueryOperationType, "query." + queryType)
	
	for mutationType in mutationOperationsTMP:
		parentMutationOperationType = copy.deepcopy(mutationOperationsTMP[mutationType])
		getChildOperations("mutation", parentMutationOperationType, parentMutationOperationType, "mutation." + mutationType)

	for operationType in catoApiSchema:
		for operationName in catoApiSchema[operationType]:
			# if operationName=="query.xdr.stories":
			childOperations = catoApiSchema[operationType][operationName]["childOperations"].keys() if "childOperations" in catoApiSchema[operationType][operationName] and catoApiSchema[operationType][operationName]!=None else []
			parsedOperation = parseOperation(catoApiSchema[operationType][operationName],childOperations)
			parsedOperation = getOperationArgs(parsedOperation["type"]["definition"],parsedOperation)
			parsedOperation["path"] = operationName
			for argName in parsedOperation["args"]:
				arg = parsedOperation["args"][argName]
				parsedOperation["operationArgs"][arg["varName"]] = arg
			parsedOperation["variablesPayload"] = generateExampleVariables(parsedOperation)
			writeFile("../models/"+operationName+".json",json.dumps(parsedOperation, indent=4, sort_keys=True))
			writeFile("../queryPayloads/"+operationName+".json",json.dumps(generateGraphqlPayload(parsedOperation["variablesPayload"],parsedOperation,operationName),indent=4,sort_keys=True))
			payload = generateGraphqlPayload(parsedOperation["variablesPayload"],parsedOperation,operationName)
			writeFile("../queryPayloads/"+operationName+".txt",payload["query"])

def getChildOperations(operationType, curType, parentType, parentPath):
	# Parse fields for nested args to map out all child operations
	# This will separate fields like stories and story for query.xdr, 
	# and all fields which are actually sub operations from mutation.internetFirewall, etc 
	if "childOperations" not in parentType:
		parentType["childOperations"] = {}
	curOfType = None
	if "kind" in curType:
		curOfType = copy.deepcopy(catoApiIntrospection[curType["kind"].lower() + "s"][curType["name"]])
	elif "type" in curType and curType["type"]["ofType"]==None:
		curOfType = copy.deepcopy(catoApiIntrospection[curType["type"]["kind"].lower() + "s"][curType["type"]["name"]])
	elif "type" in curType and curType["type"]["ofType"]["ofType"]==None:
		curOfType = copy.deepcopy(catoApiIntrospection[curType["type"]["ofType"]["kind"].lower() + "s"][curType["type"]["ofType"]["name"]])
	elif "type" in curType and curType["type"]["ofType"]["ofType"]["ofType"]==None:
		curOfType = copy.deepcopy(catoApiIntrospection[curType["type"]["ofType"]["ofType"]["kind"].lower() + "s"][curType["type"]["ofType"]["ofType"]["name"]])
	elif "type" in curType and curType["type"]["ofType"]["ofType"]["ofType"]["ofType"]==None:
		curOfType = copy.deepcopy(catoApiIntrospection[curType["type"]["ofType"]["ofType"]["ofType"]["kind"].lower() + "s"][curType["type"]["ofType"]["ofType"]["ofType"]["name"]])
	else:
		curOfType = copy.deepcopy(catoApiIntrospection[curType["type"]["ofType"]["ofType"]["ofType"]["ofType"]["kind"].lower() + "s"][curType["type"]["ofType"]["ofType"]["ofType"]["ofType"]["name"]])
	hasChildren = False

	if "fields" in curOfType and curOfType["fields"] != None:
		parentFields = []
		for field in curOfType["fields"]:
			curFieldObject = copy.deepcopy(field)
			if "args" in curFieldObject and len(curFieldObject["args"])>0:
				hasChildren = True
				curParentType = copy.deepcopy(parentType)
				curFieldObject["args"] = getNestedArgDefinitions(curFieldObject["args"], curFieldObject["name"],None,None)
				curParentType["childOperations"][curFieldObject["name"]] = curFieldObject
				getChildOperations(operationType, curFieldObject, curParentType, parentPath + "." + curFieldObject["name"])
	if not hasChildren:
		catoApiSchema[operationType][parentPath] = parentType

def getNestedArgDefinitions(argsAry, parentParamPath, childOperations, parentFields):
	newArgsList = {}
	for arg in argsAry:
		curParamPath = arg["name"] if (parentParamPath == None or parentParamPath == "") else parentParamPath.replace("___",".") + "." + arg["name"]
		if "path" in arg and '.' not in arg["path"]:
			arg["child"] = True
			arg["parent"] = arg["path"]		
		arg["type"] = getOfType(arg["type"], { "non_null": False, "kind": [], "name": None }, curParamPath, childOperations, parentFields)
		arg["path"] = curParamPath
		arg["id_str"] = curParamPath.replace(".","___")
		if isinstance(arg["type"]["kind"], list):
			arg["required"] = True if arg["type"]["kind"][0] == "NON_NULL" else False
		else:
			arg["required"] = True if arg["type"]["kind"] == "NON_NULL" else False
		required1 = "!" if arg["required"] else ""
		required2 = "!" if "NON_NULL" in arg["type"]["kind"][1:] else ""
		if "SCALAR" in arg["type"]["kind"] or "ENUM" in arg["type"]["kind"]:
			arg["varName"] = renderCamelCase(arg["name"])
			# arg["id_str"] = arg["varName"]
		else:
			arg["varName"] = renderCamelCase(arg["type"]["name"])
		arg["responseStr"] = arg["name"] + ":$" + arg["varName"] + " "
		if "LIST" in arg["type"]["kind"]:
			arg["requestStr"] = "$" + arg["varName"] + ":" + "[" + arg["type"]["name"] + required2 + "]" + required1 + " "
		else:
			arg["requestStr"] = "$" + arg["varName"] + ":" + arg["type"]["name"] + required1 + " "
		newArgsList[arg["id_str"]] = arg
	# print("getNestedArgDefinitions()",newArgsList.keys())
	return newArgsList

def getOfType(curType, ofType, parentParamPath, childOperations, parentFields):
	ofType["kind"].append(copy.deepcopy(curType["kind"]))
	curParamPath = "" if (parentParamPath == None) else parentParamPath + "___"
	if curType["ofType"] != None:
		ofType = getOfType(copy.deepcopy(curType["ofType"]), ofType, parentParamPath,childOperations,parentFields)
	else:
		ofType["name"] = curType["name"]	
	parentFields = []
	if "definition" in ofType and "fields" in ofType["definition"] and ofType["definition"]["fields"]!=None:
		for fieldName in ofType["definition"]["fields"]:
			field = ofType["definition"]["fields"][fieldName]
			parentFields.append(field["name"])			
	if "INPUT_OBJECT" in ofType["kind"]:
		ofType["indexType"] = "input_object"
		ofType["definition"] = copy.deepcopy(catoApiIntrospection["input_objects"][ofType["name"]])
		if ofType["definition"]["inputFields"] != None:
			ofType["definition"]["inputFields"] = getNestedFieldDefinitions(copy.deepcopy(ofType["definition"]["inputFields"]), curParamPath, childOperations, parentFields)
	elif "UNION" in ofType["kind"]:
		ofType["indexType"] = "interface"
		ofType["definition"] = copy.deepcopy(catoApiIntrospection["unions"][ofType["name"]])
		if ofType["definition"]["possibleTypes"] != None:
			ofType["definition"]["possibleTypes"] = getNestedInterfaceDefinitions(copy.deepcopy(ofType["definition"]["possibleTypes"]), curParamPath,childOperations, parentFields)
			# strip out each nested interface attribute from parent oftype fields, 
			# this is to prevent duplicate fields causing the query to fail
			for interfaceName in ofType["definition"]["possibleTypes"]:
				possibleType = ofType["definition"]["possibleTypes"][interfaceName]
				if ofType["definition"]["fields"]!=None:
					for fieldName in ofType["definition"]["fields"]:
						field = ofType["definition"]["fields"][fieldName]
						nestedFieldPath = parentParamPath + interfaceName + "___" + field["name"]
						# aliasLogic
						# if field["name"] in parentFields:
						# 	field["alias"] = renderCamelCase(interfaceName+"."+field["name"])+": "+field["name"]
						# 	if "SCALAR" in field["type"]["kind"] or "ENUM" in field["type"]["kind"]:
						# 		possibleType["fields"][nestedFieldPath]["alias"] = renderCamelCase(interfaceName+"."+field["name"])+": "+field["name"]
						# 	else: 
						# 		possibleType["fields"][nestedFieldPath]["alias"] = renderCamelCase(field["type"]["name"])+": "+field["name"]
				ofType["definition"]["possibleTypes"][interfaceName] = copy.deepcopy(possibleType)
	elif "OBJECT" in ofType["kind"]:
		ofType["indexType"] = "object"
		ofType["definition"] = copy.deepcopy(catoApiIntrospection["objects"][ofType["name"]])
		if ofType["definition"]["fields"] != None and childOperations!=None:
			ofType["definition"]["fields"] = checkForChildOperation(copy.deepcopy(ofType["definition"]["fields"]),childOperations)
			ofType["definition"]["fields"] = getNestedFieldDefinitions(copy.deepcopy(ofType["definition"]["fields"]), curParamPath,childOperations, parentFields)
		if ofType["definition"]["interfaces"] != None:
			ofType["definition"]["interfaces"] = getNestedInterfaceDefinitions(copy.deepcopy(ofType["definition"]["interfaces"]), curParamPath,childOperations, parentFields)
	elif "INTERFACE" in ofType["kind"]:
		ofType["indexType"] = "interface"
		ofType["definition"] = copy.deepcopy(catoApiIntrospection["interfaces"][ofType["name"]])
		if ofType["definition"]["fields"] != None:
			ofType["definition"]["fields"] = getNestedFieldDefinitions(copy.deepcopy(ofType["definition"]["fields"]), curParamPath, childOperations, parentFields)
		if ofType["definition"]["possibleTypes"] != None:
			ofType["definition"]["possibleTypes"] = getNestedInterfaceDefinitions(copy.deepcopy(ofType["definition"]["possibleTypes"]), curParamPath, childOperations, parentFields)
			for interfaceName in ofType["definition"]["possibleTypes"]:
				possibleType = copy.deepcopy(ofType["definition"]["possibleTypes"][interfaceName])
				for fieldName in ofType["definition"]["fields"]:
					field = ofType["definition"]["fields"][fieldName]
					nestedFieldPath = parentParamPath + interfaceName + "." + field["name"]
					nestedFieldPath = nestedFieldPath.replace(".","___")
					if "args" in field and len(field["args"])>0:
						field["args"] = getNestedArgDefinitions(copy.deepcopy(field["args"]), nestedFieldPath, curParamPath, parentFields)
					# aliasLogic MUST
					# CatoEndpointUser
					if field["name"] in possibleType["fields"] and possibleType["fields"][field["name"]] != None:
						# del possibleType["fields"][field["name"]]
						if "SCALAR" in field["type"]["kind"] or "ENUM" in field["type"]["kind"]:
							possibleType["fields"][field["name"]]["alias"] = renderCamelCase(interfaceName+"."+field["name"])+": "+field["name"]
						else: 
							possibleType["fields"][field["name"]]["alias"] = renderCamelCase(field["type"]["name"])+": "+field["name"]
				ofType["definition"]["possibleTypes"][interfaceName] = copy.deepcopy(possibleType)
		if ofType["definition"]["interfaces"] != None:
			ofType["definition"]["interfaces"] = getNestedInterfaceDefinitions(copy.deepcopy(ofType["definition"]["interfaces"]), curParamPath,childOperations, parentFields)
	elif "ENUM" in ofType["kind"]:
		ofType["indexType"] = "enum"
		ofType["definition"] = copy.deepcopy(catoApiIntrospection["enums"][ofType["name"]])
	return ofType

def getNestedFieldDefinitions(fieldsAry, parentParamPath,childOperations, parentFields):
	newFieldsList = {}
	for field in fieldsAry:
		if isinstance(field,str):
			field = fieldsAry[field]
		curParamPath = field["name"] if (parentParamPath == None) else (parentParamPath.replace("___",".") + field["name"])
		# curParamPath = field["name"] if (parentParamPath == None) else (parentParamPath + "." + field["name"])
		field["type"] = getOfType(field["type"], { "non_null": False, "kind": [], "name": None }, curParamPath,childOperations, parentFields)
		field["path"] = curParamPath
		field["id_str"] = curParamPath.replace(".","___")
		if isinstance(field["type"]["kind"], list):
			field["required"] = True if field["type"]["kind"][0] == "NON_NULL" else False
		else:
			field["required"] = True if field["type"]["kind"] == "NON_NULL" else False
		required1 = "!" if field["required"] else ""
		required2 = "!" if field["type"]["kind"][1:] == "NON_NULL" else ""
		if "SCALAR" in field["type"]["kind"] or "ENUM" in field["type"]["kind"]:
			field["varName"] = renderCamelCase(field["name"])
			# field["id_str"] = field["varName"]
		else:
			field["varName"] = renderCamelCase(field["type"]["name"])
		field["responseStr"] = field["name"] + ":$" + field["varName"] + " "
		if "LIST" in field["type"]["kind"]:
			field["requestStr"] = "$" + field["varName"] + ":" + "[" + field["type"]["name"] + required2 + "]" + required1 + " "
		else:
			field["requestStr"] = "$" + field["varName"] + ":" + field["type"]["name"] + required1 + " "
		if "args" in field:
			field["args"] = getNestedArgDefinitions(field["args"], field["name"],childOperations, parentFields)
		## aliasLogic must
		if parentFields!=None and field["name"] in parentFields and "SCALAR" not in field["type"]["kind"]:
			# if field["name"]=="records":
			# 	raise ValueError('A very specific bad thing happened.')
				# print(json.dumps(field,indent=2,sort_keys=True))
				# print(field["path"],parentFields)
			# field["alias"] = renderCamelCase(field["type"]["name"]+"."+field["name"])+": "+field["name"]
			field["alias"] = renderCamelCase(field["type"]["name"]+"."+field["name"])+": "+field["name"]
		if "records.fields" not in field["path"]:
			newFieldsList[field["name"]] = field	
	# for (fieldPath in newFieldList) {
	# 	var field = newFieldList[fieldPath];
	# 	if (curOperationObj.fieldTypes && field.type.name != null) curOperationObj.fieldTypes[field.type.name] = true;
	# 	field.type = getOfType(field.type, { non_null: false, kind: [], name: null }, field.path);
	# }	
	return newFieldsList

def getNestedInterfaceDefinitions(possibleTypesAry, parentParamPath,childOperations, parentFields):
	curInterfaces = {}
	for possibleType in possibleTypesAry:
		if "OBJECT" in possibleType["kind"]:
			curInterfaces[possibleType["name"]] = copy.deepcopy(catoApiIntrospection["objects"][possibleType["name"]])
	# for curInterface in curInterfaces:
	# 	curParamPath = "" if parentParamPath == None else parentParamPath + curInterface["name"] + "___"
	for curInterfaceName in curInterfaces:
		curInterface = curInterfaces[curInterfaceName]
		curParamPath = "" if parentParamPath == None else parentParamPath + curInterface["name"] + "___"
		if "fields" in curInterface and curInterface["fields"] != None:
			curInterface["fields"] = getNestedFieldDefinitions(copy.deepcopy(curInterface["fields"]), curParamPath,childOperations, parentFields)
		if "inputFields" in curInterface and curInterface["inputFields"] != None:
			curInterface["inputFields"] = getNestedFieldDefinitions(copy.deepcopy(curInterface["inputFields"]), curParamPath,childOperations, parentFields)
		if "interfaces" in curInterface and curInterface["interfaces"] != None:
			curInterface["interfaces"] = getNestedInterfaceDefinitions(copy.deepcopy(curInterface["interfaces"]), curParamPath,childOperations, parentFields)
		if "possibleTypes" in curInterface and curInterface["possibleTypes"] != None:
			curInterface["possibleTypes"] = getNestedInterfaceDefinitions(copy.deepcopy(curInterface["possibleTypes"]), curParamPath,childOperations, parentFields)
	return curInterfaces

def parseOperation(curOperation,childOperations):
	if "operationArgs" not in curOperation:
		curOperation["operationArgs"] = {}
	curOperation["fieldTypes"] = {}
	curOfType = getOfType(curOperation["type"], { "non_null": False, "kind": [], "name": None }, None,childOperations,None)
	curOperation["type"] = copy.deepcopy(curOfType)
	if curOfType["name"] in catoApiIntrospection["objects"]:
		curOperation["args"] = getNestedArgDefinitions(curOperation["args"], None,childOperations,None)
		curOperation["type"]["definition"] = copy.deepcopy(catoApiIntrospection["objects"][curOperation["type"]["name"]])		
		if "fields" in curOperation["type"]["definition"] and curOperation["type"]["definition"]["fields"] != None:
			# aliasLogic
			# parentFields = []
			# for field in curOperation["type"]["definition"]["fields"]:
			# 	parentFields.append(field["name"])
			curOperation["type"]["definition"]["fields"] = checkForChildOperation(copy.deepcopy(curOperation["type"]["definition"]["fields"]),childOperations)
			curOperation["type"]["definition"]["fields"] = copy.deepcopy(getNestedFieldDefinitions(curOperation["type"]["definition"]["fields"], None,childOperations,[]))
		if "inputFields" in curOperation["type"]["definition"] and curOperation["type"]["definition"]["inputFields"] != None:
			parentFields = curOperation["type"]["definition"]["inputFields"].keys()
			curOperation["type"]["definition"]["inputFields"] = copy.deepcopy(getNestedFieldDefinitions(curOperation["type"]["definition"]["inputFields"], None,childOperations,parentFields))
	return curOperation

def checkForChildOperation(fieldsAry,childOperations):
	newFieldList = {}
	subOperation = False
	for i, field in enumerate(fieldsAry):
		if field["name"] in childOperations:
			subOperation = field
		newFieldList[field["name"]] = copy.deepcopy(field)
	if subOperation != False:
		newFieldList = {}
		newFieldList[subOperation["name"]] = subOperation
	return newFieldList

def getOperationArgs(curType,curOperation):
	if curType.get('fields'):
		for fieldName in curType["fields"]:
			field = curType["fields"][fieldName]
			## aliasLogic
			if "type" in field and "definition" in field["type"]:
				curOperation["fieldTypes"][field["type"]["definition"]["name"]] = True
				curOperation = getOperationArgs(field["type"]["definition"],curOperation)
			if "args" in field:
				for argName in field["args"]:
					arg = field["args"][argName]
					curOperation["operationArgs"][arg["varName"]] = arg
					if "type" in arg and "definition" in arg["type"]:
						curOperation = getOperationArgs(arg["type"]["definition"],curOperation)
	if curType.get('inputFields'):
		for inputFieldName in curType["inputFields"]:
			inputField = curType["inputFields"][inputFieldName]
			if "type" in inputField and "definition" in inputField["type"]:
				curOperation["fieldTypes"][inputField["type"]["definition"]["name"]] = True
				curOperation = getOperationArgs(inputField["type"]["definition"],curOperation)
			if "args" in inputField:
				for argName in inputField["args"]:
					arg = inputField["args"][argName]
					curOperation["operationArgs"][arg["varName"]] = arg
					if "type" in arg and "definition" in arg["type"]:
						curOperation = getOperationArgs(arg["type"]["definition"],curOperation)
	if curType.get('interfaces'):
		for interface in curType["interfaces"]:
			if "type" in interface and "definition" in interface["type"]:
				curOperation["fieldTypes"][interface["type"]["definition"]["name"]] = True
				curOperation = getOperationArgs(interface["type"]["definition"],curOperation)
			if "args" in interface:
				for argName in interface["args"]:
					arg = interface["args"][argName]
					curOperation["operationArgs"][arg["varName"]] = arg
					if "type" in arg and "definition" in arg["type"]:
						curOperation = getOperationArgs(arg["type"]["definition"],curOperation)
	if curType.get('possibleTypes'):
		for possibleTypeName in curType["possibleTypes"]:
			possibleType = curType["possibleTypes"][possibleTypeName]
			curOperation = getOperationArgs(possibleType,curOperation)
			if possibleType.get('fields'):
				for fieldName in possibleType["fields"]:
					field = possibleType["fields"][fieldName]
					## aliasLogic
					# if "type" in curOperation and "definition" in curOperation["type"] and "fields" in curOperation["type"]["definition"] and field["name"] in curOperation["type"]["definition"]["fields"]:
					# 	# if field["type"]["definition"]["fields"]==None or field["type"]["definition"]["inputFields"]:
					# 	# if "SCALAR" not in field["type"]["kind"] or "ENUM" not in field["type"]["kind"]:
					# 	field["alias"] = renderCamelCase(possibleTypeName+"."+field["name"])+": "+field["name"]
			if "args" in possibleType:
				for argName in possibleType["args"]:
					arg = possibleType["args"][argName]
					curOperation["operationArgs"][arg["varName"]] = arg
					if "type" in arg and "definition" in arg["type"]:
						curOperation = getOperationArgs(arg["type"]["definition"],curOperation)
	return curOperation

def renderCamelCase(pathStr):
	str = ""
	pathAry = pathStr.split(".") 
	for i, path in enumerate(pathAry):
		if i == 0:
			str += path[0].lower() + path[1:]
		else:
			str += path[0].upper() + path[1:]
	return str	

## Functions to create sample nested variable objects for cli arguments ##
def generateExampleVariables(operation):
	variablesObj = {}
	for argName in operation["operationArgs"]:
		arg = operation["operationArgs"][argName]
		if "SCALAR" in arg["type"]["kind"] or "ENUM" in arg["type"]["kind"]:
			variablesObj[arg["name"]] = renderInputFieldVal(arg)
		else:
			argTD = arg["type"]["definition"]
			variablesObj[arg["varName"]] = {}
			if "inputFields" in argTD and argTD["inputFields"] != None:
				for inputFieldName in argTD["inputFields"]:
					inputField = argTD["inputFields"][inputFieldName]
					variablesObj[arg["varName"]][inputField["varName"]] = parseNestedArgFields(inputField)
			if "fields" in argTD and argTD["fields"] != None:
				for fieldName in argTD["fields"]:
					field = argTD["fields"][fieldName]
					variablesObj[arg["varName"]][field["varName"]] = parseNestedArgFields(field)
			if "possibleTypes" in argTD and argTD["possibleTypes"] != None:
				for possibleTypeName in argTD["possibleTypes"]:
					possibleType = argTD["possibleTypes"][possibleTypeName]
					variablesObj[arg["varName"]][possibleType["varName"]] = parseNestedArgFields(possibleTypeName)
	if "accountID" in variablesObj:
		del variablesObj["accountID"]
	if "accountId" in variablesObj:
		del variablesObj["accountId"]
	return variablesObj

def parseNestedArgFields(fieldObj):
	subVariableObj = {}
	if "SCALAR" in fieldObj["type"]["kind"] or "ENUM" in fieldObj["type"]["kind"]:
		subVariableObj[fieldObj["name"]] = renderInputFieldVal(fieldObj)
	else:
		fieldTD = fieldObj["type"]["definition"]
		if "inputFields" in fieldTD and fieldTD["inputFields"] != None:
			for inputFieldName in fieldTD["inputFields"]:
				inputField = fieldTD["inputFields"][inputFieldName]
				subVariableObj[inputField["name"]] = parseNestedArgFields(inputField)
		if "fields" in fieldTD and fieldTD["fields"] != None:
			for fieldName in fieldTD["fields"]:
				field = fieldTD["fields"][fieldName]
				subVariableObj[field["name"]] = parseNestedArgFields(field)
		if "possibleTypes" in fieldTD and fieldTD["possibleTypes"] != None:
			for possibleTypeName in fieldTD["possibleTypes"]:
				possibleType = fieldTD["possibleTypes"][possibleTypeName]
				subVariableObj[possibleType["name"]] = parseNestedArgFields(possibleTypeName)
	return subVariableObj

def renderInputFieldVal(arg):
	value = "string"
	if "SCALAR" in arg["type"]["kind"]:
		if "LIST" in arg["type"]["kind"]:
			value = [arg["type"]["name"]]
		else:
			value = arg["type"]["name"]
	elif "ENUM" in arg["type"]["kind"]:
		value = "enum("+arg["type"]["name"]+")"
		# arg["type"]["definition"]["enumValues"][0]["name"]
	return value

def writeCliDriver(catoApiSchema):
	parsersIndex = {}
	for operationType in catoApiSchema:
		for operation in catoApiSchema[operationType]:
			operationNameAry = operation.split(".")
			parsersIndex[operationNameAry[0]+"_"+operationNameAry[1]] = True
	parsers = list(parsersIndex.keys())
	cliDriverStr = """
import os
import argparse
import json
import catocli
from graphql_client import Configuration
from graphql_client.api_client import ApiException
from ..parsers.parserApiClient import get_help
import sys
sys.path.insert(0, 'vendor')
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
if "CATO_TOKEN" not in os.environ:
	print("Missing authentication, please set the CATO_TOKEN environment variable with your api key.")
	exit()
CATO_TOKEN = os.getenv("CATO_TOKEN")
CATO_DEBUG = bool(os.getenv("CATO_DEBUG", False))
from ..parsers.raw import raw_parse
from ..parsers.custom import custom_parse
from ..parsers.query_siteLocation import query_siteLocation_parse
"""
	for parserName in parsers:
		cliDriverStr += "from ..parsers."+parserName+" import "+parserName+"_parse\n"

	cliDriverStr += """
configuration = Configuration()
configuration.verify_ssl = False
configuration.api_key["x-api-key"] = CATO_TOKEN
configuration.host = "{}".format(catocli.__cato_host__)
configuration.debug = CATO_DEBUG
configuration.version = "{}".format(catocli.__version__)

parser = argparse.ArgumentParser(prog='catocli', usage='%(prog)s <operationType> <operationName> [options]', description="CLI for query on CATO via API.")
parser.add_argument('--version', action='version', version=catocli.__version__)
subparsers = parser.add_subparsers()
custom_parsers = custom_parse(subparsers)
raw_parsers = subparsers.add_parser('raw', help='Raw GraphQL', usage=get_help("raw"))
raw_parser = raw_parse(raw_parsers)
query_parser = subparsers.add_parser('query', help='Query', usage='catocli query <operationName> [options]')
query_subparsers = query_parser.add_subparsers(description='valid subcommands', help='additional help')
query_siteLocation_parser = query_siteLocation_parse(query_subparsers)
mutation_parser = subparsers.add_parser('mutation', help='Mutation', usage='catocli mutation <operationName> [options]')
mutation_subparsers = mutation_parser.add_subparsers(description='valid subcommands', help='additional help')

"""
	for parserName in parsers:
		cliDriverStr += parserName+"_parser = "+parserName+"_parse("+parserName.split("_").pop(0)+"_subparsers)\n"

	cliDriverStr += """

def main(args=None):
	args = parser.parse_args(args=args)
	try:
		response = args.func(args, configuration)

		if type(response) == ApiException:
			print("ERROR! Status code: {}".format(response.status))
			print(response)
		else:
			if response!=None:
				print(json.dumps(response[0], sort_keys=True, indent=4))
	except AttributeError:
		print('Missing arguments. Usage: catocli <operation> -h')
"""
	writeFile("../catocli/Utils/clidriver.py",cliDriverStr)

def writeOperationParsers(catoApiSchema):
	parserMapping = {"query":{},"mutation":{}}
	## Write the raw query parser ##
	cliDriverStr =f"""
from ..parserApiClient import createRawRequest, get_help

def raw_parse(raw_parser):
	raw_parser.add_argument('json', help='Query, Variables and opertaionName in JSON format.')
	raw_parser.add_argument('-t', const=True, default=False, nargs='?', help='Print test request preview without sending api call')
	raw_parser.add_argument('-v', const=True, default=False, nargs='?', help='Verbose output')
	raw_parser.add_argument('-p', const=True, default=False, nargs='?', help='Pretty print')
	raw_parser.set_defaults(func=createRawRequest,operation_name='raw')
"""
	parserPath = "../catocli/parsers/raw"
	if not os.path.exists(parserPath):
		os.makedirs(parserPath)
	writeFile(parserPath+"/__init__.py",cliDriverStr)

	## Write the siteLocation query parser ##
	cliDriverStr =f"""
from ..parserApiClient import querySiteLocation, get_help

def query_siteLocation_parse(query_subparsers):
	query_siteLocation_parser = query_subparsers.add_parser('siteLocation', 
			help='siteLocation local cli query', 
			usage=get_help("query_siteLocation"))

	query_siteLocation_parser.add_argument('accountID', help='The Account ID.')
	query_siteLocation_parser.add_argument('json', help='Variables in JSON format.')
	query_siteLocation_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	query_siteLocation_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	query_siteLocation_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	query_siteLocation_parser.set_defaults(func=querySiteLocation,operation_name='query.siteLocation')
"""
	parserPath = "../catocli/parsers/query_siteLocation"
	if not os.path.exists(parserPath):
		os.makedirs(parserPath)
	writeFile(parserPath+"/__init__.py",cliDriverStr)

	for operationType in parserMapping:
		operationAry = catoApiSchema[operationType]
		for operationName in operationAry:
			parserMapping = getParserMapping(parserMapping,operationName,operationName,operationAry[operationName])
	for operationType in parserMapping:
		for operationName in parserMapping[operationType]:
			parserName = operationType+"_"+operationName
			parser = parserMapping[operationType][operationName]
			cliDriverStr = f"""
from ..parserApiClient import createRequest, get_help

def {parserName}_parse({operationType}_subparsers):
	{parserName}_parser = {operationType}_subparsers.add_parser('{operationName}', 
			help='{operationName}() {operationType} operation', 
			usage=get_help("{operationType}_{operationName}"))
"""
			if "path" in parser:
				cliDriverStr += f"""
	{parserName}_parser.add_argument('accountID', help='The Account ID.')
	{parserName}_parser.add_argument('json', help='Variables in JSON format.')
	{parserName}_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	{parserName}_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	{parserName}_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	{parserName}_parser.set_defaults(func=createRequest,operation_name='{parserName.replace("_",".")}')
"""
			else:
				cliDriverStr += renderSubParser(parser,operationType+"_"+operationName)
			parserPath = "../catocli/parsers/"+parserName
			if not os.path.exists(parserPath):
				os.makedirs(parserPath)
			writeFile(parserPath+"/__init__.py",cliDriverStr)

def renderSubParser(subParser,parentParserPath):
	cliDriverStr = f"""
	{parentParserPath}_subparsers = {parentParserPath}_parser.add_subparsers()
"""
	for subOperationName in subParser:
		subOperation = subParser[subOperationName]
		subParserPath = parentParserPath.replace(".","_")+"_"+subOperationName
		cliDriverStr += f"""
	{subParserPath}_parser = {parentParserPath}_subparsers.add_parser('{subOperationName}', 
			help='{subOperationName}() {parentParserPath.split('_').pop()} operation', 
			usage=get_help("{subParserPath}"))
"""
		if "path" in subOperation:
			command = parentParserPath.replace("_"," ")+" "+subOperationName
			cliDriverStr += f"""
	{subParserPath}_parser.add_argument('accountID', help='The Account ID.')
	{subParserPath}_parser.add_argument('json', help='Variables in JSON format.')
	{subParserPath}_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	{subParserPath}_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	{subParserPath}_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	{subParserPath}_parser.set_defaults(func=createRequest,operation_name='{subOperation["path"]}')
"""
		else:
			cliDriverStr += renderSubParser(subOperation,subParserPath)
	return cliDriverStr

def writeReadmes(catoApiSchema):
	parserMapping = {"query":{},"mutation":{}}
	
	## Write the raw query readme ##
	readmeStr = """
## CATO-CLI - raw.graphql
[Click here](https://api.catonetworks.com/documentation/) for documentation on this operation.

### Usage for raw.graphql

`catocli raw -h`

`catocli raw <json>`

`catocli raw "$(cat < rawGraphqQL.json)"`

`catocli raw '{ "query": "query operationNameHere($yourArgument:String!) { field1 field2 }", "variables": { "yourArgument": "string", "accountID": "10949" }, "operationName": "operationNameHere" } '`

`catocli raw '{ "query": "mutation operationNameHere($yourArgument:String!) { field1 field2 }", "variables": { "yourArgument": "string", "accountID": "10949" }, "operationName": "operationNameHere" } '`
"""
	parserPath = "../catocli/parsers/raw"
	if not os.path.exists(parserPath):
		os.makedirs(parserPath)
	writeFile(parserPath+"/README.md",readmeStr)
	
		## Write the query.siteLocation readme ##
	readmeStr = """

## CATO-CLI - query.siteLocation:

### Usage for query.siteLocation:

`catocli query siteLocation -h`

`catocli query siteLocation <accountID> <json>`

`catocli query siteLocation 12345 "$(cat < siteLocation.json)"`

`catocli query siteLocation 12345 '{"filters":[{"search": "Your city here","field":"city","operation":"exact"}]}'`

`catocli query siteLocation 12345 '{"filters":[{"search": "Your Country here","field":"countryName","operation":"startsWith"}]}'`

`catocli query siteLocation 12345 '{"filters":[{"search": "Your stateName here","field":"stateName","operation":"endsWith"}]}'`

`catocli query siteLocation 12345 '{filters:[{"search": "Your City here","field":"city","operation":"startsWith"},{"search": "Your StateName here","field":"stateName","operation":"endsWith"},{"search": "Your Country here","field":"countryName","operation":"contains"}}'`

#### Operation Arguments for query.siteLocation ####
`accountID` [ID] - (optional) Unique Identifier of Account. 
`filters[]` [Array] - (optional) Array of objects consisting of `search`, `field` and `operation` attributes.
`filters[].search` [String] - (required) String to match countryName, stateName, or city specificed in `filters[].field`.
`filters[].field` [String] - (required) Specify field to match query against, defaults to look for any.  Possible values: `countryName`, `stateName`, or `city`.
`filters[].operation` [string] - (required) If a field is specified, operation to match the field value.  Possible values: `startsWith`,`endsWith`,`exact`, `contains`.
"""
	parserPath = "../catocli/parsers/query_siteLocation"
	if not os.path.exists(parserPath):
		os.makedirs(parserPath)
	writeFile(parserPath+"/README.md",readmeStr)
	
	for operationType in parserMapping:
		operationAry = catoApiSchema[operationType]
		for operationName in operationAry:
			parserMapping = getParserMapping(parserMapping,operationName,operationName,operationAry[operationName])
	for operationType in parserMapping:
		for operationName in parserMapping[operationType]:
			parserName = operationType+"_"+operationName
			parser = parserMapping[operationType][operationName]
			operationPath = operationType+"."+operationName
			operationCmd = operationType+" "+operationName
			readmeStr = f"""
## CATO-CLI - {operationPath}:
[Click here](https://api.catonetworks.com/documentation/#{operationType}-{operationName}) for documentation on this operation.

### Usage for {operationPath}:

`catocli {operationCmd} -h`
"""
			if "path" in parser:
				readmeStr += f"""
`catocli {operationCmd} <accountID> <json>`

`catocli {operationCmd} 12345 "$(cat < {operationName}.json)"`

`catocli {operationCmd} 12345 '{json.dumps(parser["example"])}'`

#### Operation Arguments for {operationPath} ####
"""
				for argName in parser["args"]:
					arg = parser["args"][argName]
					readmeStr += '`'+argName+'` ['+arg["type"]+'] - '
					readmeStr += '('+arg["required"]+') '+arg["description"]+' '
					readmeStr += 'Default Value: '+str(arg["values"]) if len(arg["values"])>0 else ""
					readmeStr += "\n"
				parserPath = "../catocli/parsers/"+parserName
				if not os.path.exists(parserPath):
					os.makedirs(parserPath)
				writeFile(parserPath+"/README.md",readmeStr)
			else:
				parserPath = "../catocli/parsers/"+parserName
				if not os.path.exists(parserPath):
					os.makedirs(parserPath)
				writeFile(parserPath+"/README.md",readmeStr)
				renderSubReadme(parser,operationType,operationType+"."+operationName)

def renderSubReadme(subParser,operationType,parentOperationPath):
	for subOperationName in subParser:
		subOperation = subParser[subOperationName]
		subOperationPath = parentOperationPath+"."+subOperationName
		subOperationCmd = parentOperationPath.replace("."," ")+" "+subOperationName
		parserPath = "../catocli/parsers/"+subOperationPath.replace(".","_")
		readmeStr = f"""
## CATO-CLI - {parentOperationPath}.{subOperationName}:
[Click here](https://api.catonetworks.com/documentation/#{operationType}-{subOperationName}) for documentation on this operation.

### Usage for {subOperationPath}:

`catocli {subOperationCmd} -h`
"""
		if "path" in subOperation:
			readmeStr += f"""
`catocli {subOperationCmd} <accountID> <json>`

`catocli {subOperationCmd} 12345 "$(cat < {subOperationName}.json)"`

`catocli {subOperationCmd} 12345 '{json.dumps(subOperation["example"])}'`

#### Operation Arguments for {subOperationPath} ####
"""
			for argName in subOperation["args"]:
				arg = subOperation["args"][argName]
				readmeStr += '`'+argName+'` ['+arg["type"]+'] - '
				readmeStr += '('+arg["required"]+') '+arg["description"]+' '
				readmeStr += 'Default Value: '+str(arg["values"]) if len(arg["values"])>0 else ""
				readmeStr += "\n"
			if not os.path.exists(parserPath):
				os.makedirs(parserPath)
			writeFile(parserPath+"/README.md",readmeStr)
		else:
			if not os.path.exists(parserPath):
				os.makedirs(parserPath)
			writeFile(parserPath+"/README.md",readmeStr)
			renderSubReadme(subOperation,operationType,subOperationPath)

def getParserMapping(curParser,curPath,operationFullPath,operation):
	parserObj = {
		"path":operationFullPath,
		"args":{},
		# "example":"N/A",
		"example":operation["variablesPayload"]
	}
	for argName in operation["operationArgs"]:
		arg = operation["operationArgs"][argName]
		values = []
		if "definition" in arg["type"] and "enumValues" in arg["type"]["definition"] and arg["type"]["definition"]["enumValues"]!=None:
			for enumValue in arg["type"]["definition"]["enumValues"]:
				values.append(enumValue["name"])
		parserObj["args"][arg["varName"]] = {
			"name":arg["name"],
			"description":"N/A" if arg["description"]==None else arg["description"],
			"type":arg["type"]["name"]+("[]" if "LIST" in arg["type"]["kind"] else ""),
			"required": "required" if arg["required"]==True else "optional",
			"values":values
		}
	pAry = curPath.split(".")
	pathCount = len(curPath.split("."))
	if pAry[0] not in curParser:
		curParser[pAry[0]] = {}
	if pathCount == 2:
		curParser[pAry[0]][pAry[1]] = parserObj
	else:
		if pAry[1] not in curParser[pAry[0]]:
			curParser[pAry[0]][pAry[1]] = {}
		if pathCount == 3:
			curParser[pAry[0]][pAry[1]][pAry[2]] = parserObj
		else:
			if pAry[2] not in curParser[pAry[0]][pAry[1]]:
				curParser[pAry[0]][pAry[1]][pAry[2]] = {}
			if pathCount == 4:
				curParser[pAry[0]][pAry[1]][pAry[2]][pAry[3]] = parserObj
			else:
				if pAry[3] not in curParser[pAry[0]][pAry[1]][pAry[2]]:
					curParser[pAry[0]][pAry[1]][pAry[2]][pAry[3]] = {}
				if pathCount == 5:
					curParser[pAry[0]][pAry[1]][pAry[2]][pAry[3]][pAry[4]] = parserObj
				else:
					if pAry[4] not in curParser[pAry[0]][pAry[1]][pAry[2]][pAry[3]]:
						curParser[pAry[0]][pAry[1]][pAry[2]][pAry[3]][pAry[4]] = {}
					if pathCount == 6:
						curParser[pAry[0]][pAry[1]][pAry[2]][pAry[3]][pAry[4]][pAry[5]] = parserObj
	return curParser

def send(api_key,query,variables={},operationName=None):
	headers = { 'x-api-key': api_key,'Content-Type':'application/json'}
	no_verify = ssl._create_unverified_context()
	request = urllib.request.Request(url='https://api.catonetworks.com/api/v1/graphql2',
		data=json.dumps(query).encode("ascii"),headers=headers)
	response = urllib.request.urlopen(request, context=no_verify, timeout=60)
	result_data = response.read()
	result = json.loads(result_data)
	if "errors" in result:
		logging.warning(f"API error: {result_data}")
		return False,result
	return True,result


################### adding functions local to generate dynamic payloads ####################
def generateGraphqlPayload(variablesObj,operation,operationName):
	indent = "	"
	queryStr = ""
	variableStr = ""
	for varName in variablesObj:
		if (varName in operation["operationArgs"]):
			variableStr += operation["operationArgs"][varName]["requestStr"]
	operationAry = operationName.split(".")
	operationType = operationAry.pop(0)
	queryStr = operationType + " "
	queryStr += renderCamelCase(".".join(operationAry))
	queryStr += " ( " + variableStr + ") {\n"
	queryStr += indent + operation["name"] + " ( "			
	for argName in operation["args"]:
		arg = operation["args"][argName]
		if arg["varName"] in variablesObj:
			queryStr += arg["responseStr"]
	queryStr += ") {\n" + renderArgsAndFields("", variablesObj, operation, operation["type"]["definition"], "		") + "	}"
	queryStr += indent + "\n}";
	body = {
		"query":queryStr.replace("\n", " ").replace("\t", " ").replace("    ", " ").replace("   ", " ").replace("  ", " "),
		"variables":variablesObj,
		"operationName":renderCamelCase(".".join(operationAry)),
	}
	return body

def renderCamelCase(pathStr):
	str = "";
	pathAry = pathStr.split(".") 
	for i, path in enumerate(pathAry):
		if i == 0:
			str += path
		else:
			str += path[0].upper() + path[1:]
	return str	

def renderArgsAndFields(responseArgStr, variablesObj, curOperation, definition, indent):
	for fieldName in definition['fields']:
		field = definition['fields'][fieldName]
		field_name = field['alias'] if 'alias' in field else field['name']				
		responseArgStr += indent + field_name
		if field.get("args") and not isinstance(field['args'], list):
			if (len(list(field['args'].keys()))>0):
				argsPresent = False
				argStr = " ( "
				for argName in field['args']:
					arg = field['args'][argName]
					if arg["varName"] in variablesObj:
						argStr += arg['responseStr'] + " "
						argsPresent = True
				argStr += ") "
				if argsPresent==True:
					responseArgStr += argStr
		if field.get("type") and field['type'].get('definition') and field['type']['definition']['fields'] is not None:
			responseArgStr += " {\n"
			for subfieldIndex in field['type']['definition']['fields']:
				subfield = field['type']['definition']['fields'][subfieldIndex]
				subfield_name = subfield['alias'] if 'alias' in subfield else subfield['name']				
				responseArgStr += indent + "	" + subfield_name
				if subfield.get("args") and len(list(subfield["args"].keys()))>0:
					argsPresent = False
					subArgStr = " ( "
					for argName in subfield['args']:
						arg = subfield['args'][argName]
						if arg["varName"] in variablesObj:
							argsPresent = True
							subArgStr += arg['responseStr'] + " "
					subArgStr += " )"
					if argsPresent==True:
						responseArgStr += subArgStr
				if subfield.get("type") and subfield['type'].get("definition") and (subfield['type']['definition'].get("fields") or subfield['type']['definition'].get('inputFields')):
					responseArgStr += " {\n"
					responseArgStr = renderArgsAndFields(responseArgStr, variablesObj, curOperation, subfield['type']['definition'], indent + "		")
					if subfield['type']['definition'].get('possibleTypes'):
						for possibleTypeName in subfield['type']['definition']['possibleTypes']:
							possibleType = subfield['type']['definition']['possibleTypes'][possibleTypeName]
							responseArgStr += indent + "		... on " + possibleType['name'] + " {\n"
							if possibleType.get('fields') or possibleType.get('inputFields'):
								responseArgStr = renderArgsAndFields(responseArgStr, variablesObj, curOperation, possibleType, indent + "			")
							responseArgStr += indent + "		}\n"
					responseArgStr += indent + "	}"
				elif subfield.get('type') and subfield['type'].get('definition') and subfield['type']['definition'].get('possibleTypes'):
					responseArgStr += " {\n"
					responseArgStr += indent + "		__typename\n"
					for possibleTypeName in subfield['type']['definition']['possibleTypes']:
						possibleType = subfield['type']['definition']['possibleTypes'][possibleTypeName]						
						responseArgStr += indent + "		... on " + possibleType['name'] + " {\n"
						if possibleType.get('fields') or possibleType.get('inputFields'):
							responseArgStr = renderArgsAndFields(responseArgStr, variablesObj, curOperation, possibleType, indent + "			")
						responseArgStr += indent + "		}\n"
					responseArgStr += indent + " 	}\n"
				responseArgStr += "\n"
			if field['type']['definition'].get('possibleTypes'):
				for possibleTypeName in field['type']['definition']['possibleTypes']:
					possibleType = field['type']['definition']['possibleTypes'][possibleTypeName]
					responseArgStr += indent + "	... on " + possibleType['name'] + " {\n"
					if possibleType.get('fields') or possibleType.get('inputFields'):
						responseArgStr = renderArgsAndFields(responseArgStr, variablesObj, curOperation, possibleType, indent + "		")
					responseArgStr += indent + "	}\n"
			responseArgStr += indent + "}\n"
		if field.get('type') and field['type'].get('definition') and field['type']['definition'].get('inputFields'):
			responseArgStr += " {\n"
			for subfieldName in field['type']['definition'].get('inputFields'):
				subfield = field['type']['definition']['inputFields'][subfieldName]
				subfield_name = subfield['alias'] if 'alias' in subfield else subfield['name']
				responseArgStr += indent + "	" + subfield_name
				if subfield.get('type') and subfield['type'].get('definition') and (subfield['type']['definition'].get('fields') or subfield['type']['definition'].get('inputFields')):
					responseArgStr += " {\n"
					responseArgStr = renderArgsAndFields(responseArgStr, variablesObj, curOperation, subfield['type']['definition'], indent + "		")
					responseArgStr += indent + "	}\n"
			if field['type']['definition'].get('possibleTypes'):
				for possibleTypeName in field['type']['definition']['possibleTypes']:
					possibleType = field['type']['definition']['possibleTypes'][possibleTypeName]
					responseArgStr += indent + "... on " + possibleType['name'] + " {\n"
					if possibleType.get('fields') or possibleType.get('inputFields'):
						responseArgStr = renderArgsAndFields(responseArgStr, variablesObj, curOperation, possibleType, indent + "		")
					responseArgStr += indent + "	}\n"
			responseArgStr += indent + "}\n"
		responseArgStr += "\n"
	return responseArgStr



# coding: utf-8

from __future__ import absolute_import

import io
import json
import logging
import re
import ssl

# python 2 and python 3 compatibility library
import sys
sys.path.insert(0, 'vendor')
import certifi
import six
from six.moves.urllib.parse import urlencode

try:
	import urllib3
except ImportError:
	raise ImportError('GraphQL python client requires urllib3.')

logger = logging.getLogger(__name__)

class QUERYResponse(io.IOBase):

	def __init__(self, resp):
		self.urllib3_response = resp
		self.status = resp.status
		self.reason = resp.reason
		self.data = resp.data

	def getheaders(self):
		"""Returns a dictionary of the response headers."""
		return self.urllib3_response.getheaders()

	def getheader(self, name, default=None):
		"""Returns a given response header."""
		return self.urllib3_response.getheader(name, default)

class QUERYClientObject(object):

	def __init__(self, configuration, pools_size=4, maxsize=None):
		# urllib3.PoolManager will pass all kw parameters to connectionpool
		# https://github.com/shazow/urllib3/blob/f9409436f83aeb79fbaf090181cd81b784f1b8ce/urllib3/poolmanager.py#L75  # noqa: E501
		# https://github.com/shazow/urllib3/blob/f9409436f83aeb79fbaf090181cd81b784f1b8ce/urllib3/connectionpool.py#L680  # noqa: E501
		# maxsize is the number of requests to host that are allowed in parallel  # noqa: E501
		# Custom SSL certificates and client certificates: http://urllib3.readthedocs.io/en/latest/advanced-usage.html  # noqa: E501

		# cert_reqs
		if configuration.verify_ssl:
			cert_reqs = ssl.CERT_REQUIRED
		else:
			cert_reqs = ssl.CERT_NONE

		# ca_certs
		if configuration.ssl_ca_cert:
			ca_certs = configuration.ssl_ca_cert
		else:
			# if not set certificate file, use Mozilla's root certificates.
			ca_certs = certifi.where()

		addition_pool_args = {}
		if configuration.assert_hostname is not None:
			addition_pool_args['assert_hostname'] = configuration.assert_hostname  # noqa: E501

		if maxsize is None:
			if configuration.connection_pool_maxsize is not None:
				maxsize = configuration.connection_pool_maxsize
			else:
				maxsize = 4

		# https pool manager
		if configuration.proxy:
			self.pool_manager = urllib3.ProxyManager(
				num_pools=pools_size,
				maxsize=maxsize,
				cert_reqs=cert_reqs,
				ca_certs=ca_certs,
				cert_file=configuration.cert_file,
				key_file=configuration.key_file,
				proxy_url=configuration.proxy,
				**addition_pool_args
			)
		else:
			self.pool_manager = urllib3.PoolManager(
				num_pools=pools_size,
				maxsize=maxsize,
				cert_reqs=cert_reqs,
				ca_certs=ca_certs,
				cert_file=configuration.cert_file,
				key_file=configuration.key_file,
				**addition_pool_args
			)

	def request(self, method, url, headers=None,
				body=None, _preload_content=True,
				_request_timeout=None):
		"""Perform requests.
		:param headers: http request headers
		:param body: request json body, for `application/json`
		:param _preload_content: if False, the urllib3.HTTPResponse object will
								 be returned without reading/decoding response
								 data. Default is True.
		:param _request_timeout: timeout setting for this request. If one
								 number provided, it will be total request
								 timeout. It can also be a pair (tuple) of
								 (connection, read) timeouts.
		"""
		method = method.upper()
		assert method in ['POST']
		headers = headers or {}

		timeout = None
		if _request_timeout:
			if isinstance(_request_timeout, (int, ) if six.PY3 else (int, long)):  # noqa: E501,F821
				timeout = urllib3.Timeout(total=_request_timeout)
			elif (isinstance(_request_timeout, tuple) and
				  len(_request_timeout) == 2):
				timeout = urllib3.Timeout(
					connect=_request_timeout[0], read=_request_timeout[1])

		if 'Content-Type' not in headers:
			headers['Content-Type'] = 'application/json'

		try:
			if re.search('json', headers['Content-Type'], re.IGNORECASE):
				try:
					r = self.pool_manager.request( method=method, url=url,
						body=json.dumps(body),
						preload_content=_preload_content,
						timeout=timeout,
						headers=headers)
				except Exception as e:
					# remove the restriction for raw 
					print("ERROR: ",e)
					exit()
				
			else:
				# Cannot generate the request from given parameters
				msg = """Cannot prepare a request message for provided
						 arguments. Please check that your arguments match
						 declared content type."""
				raise ApiException(status=0, reason=msg)
			
		except urllib3.exceptions.SSLError as e:
			msg = "{0}\n{1}".format(type(e).__name__, str(e))
			raise ApiException(status=0, reason=msg)

		if _preload_content:
			r = QUERYResponse(r)

			# log response body
			logger.debug("response body: %s", r.data)

		if not 200 <= r.status <= 299:
			raise ApiException(http_resp=r)

		return r

	def POST(self, url, headers=None, body=None, _preload_content=True, _request_timeout=None):
		return self.request("POST", url, headers=headers,
							_preload_content=_preload_content,
							_request_timeout=_request_timeout,
							body=body)

class ApiException(Exception):

	def __init__(self, status=None, reason=None, http_resp=None):
		if http_resp:
			self.status = http_resp.status
			self.reason = http_resp.reason
			self.body = http_resp.data
			self.headers = http_resp.getheaders()
		else:
			self.status = status
			self.reason = reason
			self.body = None
			self.headers = None

	def __str__(self):
		"""Custom error messages for exception"""
		if self.body:
			# error_message += "HTTP response body:\n{}".format(self.body.decode('utf8'))
			return self.body.decode('utf8')
		else:
			error_message = "({0})\n"\
							"Reason: {1}\n".format(self.status, self.reason)
			if self.headers:
				error_message += "HTTP response headers: {0}\n".format(
					self.headers)





from ..parserApiClient import createRequest, get_help

def mutation_site_parse(mutation_subparsers):
	mutation_site_parser = mutation_subparsers.add_parser('site', 
			help='site() mutation operation', 
			usage=get_help("mutation_site"))

	mutation_site_subparsers = mutation_site_parser.add_subparsers()

	mutation_site_addIpsecIkeV2Site_parser = mutation_site_subparsers.add_parser('addIpsecIkeV2Site', 
			help='addIpsecIkeV2Site() site operation', 
			usage=get_help("mutation_site_addIpsecIkeV2Site"))

	mutation_site_addIpsecIkeV2Site_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_addIpsecIkeV2Site_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_addIpsecIkeV2Site_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_addIpsecIkeV2Site_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_addIpsecIkeV2Site_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_addIpsecIkeV2Site_parser.set_defaults(func=createRequest,operation_name='mutation.site.addIpsecIkeV2Site')

	mutation_site_addIpsecIkeV2SiteTunnels_parser = mutation_site_subparsers.add_parser('addIpsecIkeV2SiteTunnels', 
			help='addIpsecIkeV2SiteTunnels() site operation', 
			usage=get_help("mutation_site_addIpsecIkeV2SiteTunnels"))

	mutation_site_addIpsecIkeV2SiteTunnels_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_addIpsecIkeV2SiteTunnels_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_addIpsecIkeV2SiteTunnels_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_addIpsecIkeV2SiteTunnels_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_addIpsecIkeV2SiteTunnels_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_addIpsecIkeV2SiteTunnels_parser.set_defaults(func=createRequest,operation_name='mutation.site.addIpsecIkeV2SiteTunnels')

	mutation_site_addNetworkRange_parser = mutation_site_subparsers.add_parser('addNetworkRange', 
			help='addNetworkRange() site operation', 
			usage=get_help("mutation_site_addNetworkRange"))

	mutation_site_addNetworkRange_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_addNetworkRange_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_addNetworkRange_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_addNetworkRange_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_addNetworkRange_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_addNetworkRange_parser.set_defaults(func=createRequest,operation_name='mutation.site.addNetworkRange')

	mutation_site_addSocketSite_parser = mutation_site_subparsers.add_parser('addSocketSite', 
			help='addSocketSite() site operation', 
			usage=get_help("mutation_site_addSocketSite"))

	mutation_site_addSocketSite_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_addSocketSite_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_addSocketSite_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_addSocketSite_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_addSocketSite_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_addSocketSite_parser.set_defaults(func=createRequest,operation_name='mutation.site.addSocketSite')

	mutation_site_addStaticHost_parser = mutation_site_subparsers.add_parser('addStaticHost', 
			help='addStaticHost() site operation', 
			usage=get_help("mutation_site_addStaticHost"))

	mutation_site_addStaticHost_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_addStaticHost_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_addStaticHost_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_addStaticHost_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_addStaticHost_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_addStaticHost_parser.set_defaults(func=createRequest,operation_name='mutation.site.addStaticHost')

	mutation_site_removeNetworkRange_parser = mutation_site_subparsers.add_parser('removeNetworkRange', 
			help='removeNetworkRange() site operation', 
			usage=get_help("mutation_site_removeNetworkRange"))

	mutation_site_removeNetworkRange_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_removeNetworkRange_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_removeNetworkRange_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_removeNetworkRange_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_removeNetworkRange_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_removeNetworkRange_parser.set_defaults(func=createRequest,operation_name='mutation.site.removeNetworkRange')

	mutation_site_removeSite_parser = mutation_site_subparsers.add_parser('removeSite', 
			help='removeSite() site operation', 
			usage=get_help("mutation_site_removeSite"))

	mutation_site_removeSite_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_removeSite_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_removeSite_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_removeSite_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_removeSite_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_removeSite_parser.set_defaults(func=createRequest,operation_name='mutation.site.removeSite')

	mutation_site_removeStaticHost_parser = mutation_site_subparsers.add_parser('removeStaticHost', 
			help='removeStaticHost() site operation', 
			usage=get_help("mutation_site_removeStaticHost"))

	mutation_site_removeStaticHost_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_removeStaticHost_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_removeStaticHost_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_removeStaticHost_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_removeStaticHost_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_removeStaticHost_parser.set_defaults(func=createRequest,operation_name='mutation.site.removeStaticHost')

	mutation_site_updateHa_parser = mutation_site_subparsers.add_parser('updateHa', 
			help='updateHa() site operation', 
			usage=get_help("mutation_site_updateHa"))

	mutation_site_updateHa_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_updateHa_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_updateHa_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_updateHa_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_updateHa_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_updateHa_parser.set_defaults(func=createRequest,operation_name='mutation.site.updateHa')

	mutation_site_updateIpsecIkeV2SiteGeneralDetails_parser = mutation_site_subparsers.add_parser('updateIpsecIkeV2SiteGeneralDetails', 
			help='updateIpsecIkeV2SiteGeneralDetails() site operation', 
			usage=get_help("mutation_site_updateIpsecIkeV2SiteGeneralDetails"))

	mutation_site_updateIpsecIkeV2SiteGeneralDetails_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_updateIpsecIkeV2SiteGeneralDetails_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_updateIpsecIkeV2SiteGeneralDetails_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_updateIpsecIkeV2SiteGeneralDetails_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_updateIpsecIkeV2SiteGeneralDetails_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_updateIpsecIkeV2SiteGeneralDetails_parser.set_defaults(func=createRequest,operation_name='mutation.site.updateIpsecIkeV2SiteGeneralDetails')

	mutation_site_updateIpsecIkeV2SiteTunnels_parser = mutation_site_subparsers.add_parser('updateIpsecIkeV2SiteTunnels', 
			help='updateIpsecIkeV2SiteTunnels() site operation', 
			usage=get_help("mutation_site_updateIpsecIkeV2SiteTunnels"))

	mutation_site_updateIpsecIkeV2SiteTunnels_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_updateIpsecIkeV2SiteTunnels_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_updateIpsecIkeV2SiteTunnels_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_updateIpsecIkeV2SiteTunnels_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_updateIpsecIkeV2SiteTunnels_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_updateIpsecIkeV2SiteTunnels_parser.set_defaults(func=createRequest,operation_name='mutation.site.updateIpsecIkeV2SiteTunnels')

	mutation_site_updateNetworkRange_parser = mutation_site_subparsers.add_parser('updateNetworkRange', 
			help='updateNetworkRange() site operation', 
			usage=get_help("mutation_site_updateNetworkRange"))

	mutation_site_updateNetworkRange_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_updateNetworkRange_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_updateNetworkRange_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_updateNetworkRange_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_updateNetworkRange_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_updateNetworkRange_parser.set_defaults(func=createRequest,operation_name='mutation.site.updateNetworkRange')

	mutation_site_updateSiteGeneralDetails_parser = mutation_site_subparsers.add_parser('updateSiteGeneralDetails', 
			help='updateSiteGeneralDetails() site operation', 
			usage=get_help("mutation_site_updateSiteGeneralDetails"))

	mutation_site_updateSiteGeneralDetails_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_updateSiteGeneralDetails_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_updateSiteGeneralDetails_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_updateSiteGeneralDetails_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_updateSiteGeneralDetails_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_updateSiteGeneralDetails_parser.set_defaults(func=createRequest,operation_name='mutation.site.updateSiteGeneralDetails')

	mutation_site_updateSocketInterface_parser = mutation_site_subparsers.add_parser('updateSocketInterface', 
			help='updateSocketInterface() site operation', 
			usage=get_help("mutation_site_updateSocketInterface"))

	mutation_site_updateSocketInterface_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_updateSocketInterface_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_updateSocketInterface_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_updateSocketInterface_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_updateSocketInterface_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_updateSocketInterface_parser.set_defaults(func=createRequest,operation_name='mutation.site.updateSocketInterface')

	mutation_site_updateStaticHost_parser = mutation_site_subparsers.add_parser('updateStaticHost', 
			help='updateStaticHost() site operation', 
			usage=get_help("mutation_site_updateStaticHost"))

	mutation_site_updateStaticHost_parser.add_argument('accountID', help='The Account ID.')
	mutation_site_updateStaticHost_parser.add_argument('json', help='Variables in JSON format.')
	mutation_site_updateStaticHost_parser.add_argument('-t', const=True, default=False, nargs='?', 
		help='Print test request preview without sending api call')
	mutation_site_updateStaticHost_parser.add_argument('-v', const=True, default=False, nargs='?', 
		help='Verbose output')
	mutation_site_updateStaticHost_parser.add_argument('-p', const=True, default=False, nargs='?', 
		help='Pretty print')
	mutation_site_updateStaticHost_parser.set_defaults(func=createRequest,operation_name='mutation.site.updateStaticHost')
